using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using Railroads.ScenarioManager.Core;


namespace Railroads.ScenarioManager
{
	public partial class CityManagerControl : UserControl
	{
		private bool m_bSongsUpdated;
		private bool m_bIgnoreEvents = false;
		private Scenario m_oSelectedScenario;
		private ScenarioCity m_oSelectedCity;

		
		
		public CityManagerControl()
		{
			InitializeComponent();
		}



		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public ScenarioCity SelectedCity { get { return m_oSelectedCity; } set { m_oSelectedCity = value; } }

		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public Scenario SelectedScenario 
		{
			get { return m_oSelectedScenario; }
			set 
			{
				m_oSelectedScenario = value;
				if (m_oSelectedScenario == null) ClearCity(); else LoadCities(); 
			} 
		}

		private ListViewItem SelectedItem { get { return (lvwCities.SelectedItems.Count > 0) ? lvwCities.SelectedItems[0] : null; } }


		private void lvwCities_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (lvwCities.SelectedItems.Count > 0)
			{
				using (new CursorChanger(Cursors.WaitCursor))
				{
					SaveCity();

					ScenarioCity oCity = lvwCities.SelectedItems[0].Tag as ScenarioCity;
					if (oCity != null)
					{
						m_oSelectedCity = oCity;
						DisplayCity(oCity);
					}
				}
			}
		}

		private void cboSong_SelectedValueChanged(object sender, EventArgs e)
		{
			m_bSongsUpdated = true;
		}



		public void LoadCombos()
		{
			cboEarlyEraSong.Sorted = false;
			cboMiddleEraSong.Sorted = false;
			cboLateEraSong.Sorted = false;

			cboEarlyEraSong.Items.Add("");
			cboMiddleEraSong.Items.Add("");
			cboLateEraSong.Items.Add("");

			foreach (KeyValuePair<string, Sound3d> oPair in Manager.Instance.Sounds3d)
			{
				cboEarlyEraSong.Items.Add(oPair.Value);
				cboMiddleEraSong.Items.Add(oPair.Value);
				cboLateEraSong.Items.Add(oPair.Value);
			}

			cboEarlyEraSong.Sorted = false;
			cboMiddleEraSong.Sorted = false;
			cboLateEraSong.Sorted = false;
		}

		private void ClearCity()
		{
			txtCityName.Text = "";
			txtCityTag.Text = "";
			cboCityInitialType.Text = "";
			chkCityStartLocation.Checked = false;
			txtCityStartX.Text = "";
			txtCityStartY.Text = "";
			txtCityRotation.Text = "";
			cboCityTrackSide.Text = "";

			cboEarlyEraSong.Text = "";
			cboMiddleEraSong.Text = "";
			cboLateEraSong.Text = "";
		}

		private void LoadCities()
		{
			FormsHelper.PopulateCityTypesCombo(cboCityInitialType, m_oSelectedScenario, false);
			
			ListViewItem lvi;
			lvwCities.Items.Clear();

			lvwCities.Sorting = SortOrder.None;
			
			foreach (ScenarioCity oCity in m_oSelectedScenario.Cities.Values)
			{
				lvi = new ListViewItem(oCity.Name);
				lvi.SubItems.Add(oCity.Type);
				lvi.SubItems.Add(oCity.IsStartLocation ? "Yes" : "");
				lvi.Tag = oCity;

				lvwCities.Items.Add(lvi);
			}

			lvwCities.Sorting = SortOrder.Ascending;
			
			if (lvwCities.Items.Count > 0)
			{
				lvwCities.Items[0].Selected = true;
			}
		}

		public void SaveCity()
		{
			if (m_oSelectedCity == null) return;

			ScenarioCityType oCityType = cboCityInitialType.SelectedItem as ScenarioCityType;
			if (oCityType != null)
			{
				m_oSelectedCity.Type = oCityType.Name;
			}
			m_oSelectedCity.IsStartLocation = chkCityStartLocation.Checked;

			int nTemp;
			if (int.TryParse(txtCityStartX.Text, out nTemp))
			{
				m_oSelectedCity.StartX = nTemp;
			}
			if (int.TryParse(txtCityStartY.Text, out nTemp))
			{
				m_oSelectedCity.StartY = nTemp;
			}

			double dTemp;
			if (double.TryParse(txtCityRotation.Text, out dTemp))
			{
				m_oSelectedCity.Rotation = dTemp;
			}

			try
			{
				TrackSideType oTrackSide = (TrackSideType)Enum.Parse(typeof(TrackSideType), cboCityTrackSide.Text);
				m_oSelectedCity.DefaultTrackSide = oTrackSide;
			}
			catch
			{
			}

			if (m_bSongsUpdated)
			{
				SaveSongs();
			}
		}

		private void DisplayCity(ScenarioCity city)
		{
			m_bIgnoreEvents = true;

			txtCityName.Text = city.Name;
			txtCityTag.Text = city.Tag;
			cboCityInitialType.Text = city.Type;
			chkCityStartLocation.Checked = city.IsStartLocation;
			txtCityStartX.Text = city.StartX.ToString();
			txtCityStartY.Text = city.StartY.ToString();
			txtCityRotation.Text = city.Rotation.ToString();
			cboCityTrackSide.Text = city.DefaultTrackSide.ToString();

			cboEarlyEraSong.Text = "";
			cboMiddleEraSong.Text = "";
			cboLateEraSong.Text = "";

			foreach (ScenarioCity.CitySong oSong in city.CitySongs)
			{
				foreach (string era in oSong.Eras)
				{
					switch (era)
					{
						case Constants.ERA_EARLY:	cboEarlyEraSong.Text = oSong.Song.ScriptID; break;
						case Constants.ERA_MIDDLE:	cboMiddleEraSong.Text = oSong.Song.ScriptID; break;
						case Constants.ERA_LATE:	cboLateEraSong.Text = oSong.Song.ScriptID; break;
					}
				}
			}

			m_bSongsUpdated = false;
			m_bIgnoreEvents = false;
		}

		private void SaveSongs()
		{
			ScenarioCity.CitySong oCitySong;
			m_oSelectedCity.CitySongs.Clear();
			Sound3d oSong = cboEarlyEraSong.SelectedItem as Sound3d;
			if (oSong != null)
			{
				oCitySong = new ScenarioCity.CitySong(m_oSelectedScenario.ScenarioSettings);

				oCitySong.Song = oSong;
				oCitySong.Eras.Add(Constants.ERA_EARLY);

				m_oSelectedCity.CitySongs.Add(oCitySong);
			}

			oSong = cboMiddleEraSong.SelectedItem as Sound3d;
			if (oSong != null)
			{
				oCitySong = m_oSelectedCity.CitySongs.Find(new Predicate<ScenarioCity.CitySong>(
															delegate(ScenarioCity.CitySong s)
															{
																return s.Song == oSong;
															})
					);

				if (oCitySong == null)
				{
					oCitySong = new ScenarioCity.CitySong(m_oSelectedScenario.ScenarioSettings);
					oCitySong.Song = oSong;

					m_oSelectedCity.CitySongs.Add(oCitySong);
				}

				oCitySong.Eras.Add(Constants.ERA_MIDDLE);
			}

			oSong = cboLateEraSong.SelectedItem as Sound3d;
			if (oSong != null)
			{
				oCitySong = m_oSelectedCity.CitySongs.Find(new Predicate<ScenarioCity.CitySong>(
															delegate(ScenarioCity.CitySong s)
															{
																return s.Song == oSong;
															})
					);

				if (oCitySong == null)
				{
					oCitySong = new ScenarioCity.CitySong(m_oSelectedScenario.ScenarioSettings);
					oCitySong.Song = oSong;

					m_oSelectedCity.CitySongs.Add(oCitySong);
				}

				oCitySong.Eras.Add(Constants.ERA_LATE);
			}

			m_oSelectedScenario.IsDirty = true;
		}

		private void cboCityInitialType_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;

			ScenarioCityType oCityType = cboCityInitialType.SelectedItem as ScenarioCityType;
			if (oCityType != null)
			{
				SelectedCity.Type = oCityType.Name;
				SelectedItem.SubItems[1].Text = oCityType.Name;
			}
		}

		private void chkCityStartLocation_Click(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;

			SelectedCity.IsStartLocation = chkCityStartLocation.Checked;
			SelectedItem.SubItems[2].Text = (m_oSelectedCity.IsStartLocation ? "Yes" : "");
		}
	}
}
