using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using Railroads.ScenarioManager.Core;


namespace Railroads.ScenarioManager.Editors
{
	public partial class OwnStuffConditionEditor : ConditionEditorBase
	{
		private OwnStuffCondition m_oCondition = null;

		public OwnStuffConditionEditor()
		{
			InitializeComponent();
		}


		protected override void OnSetNode(object node)
		{
			base.OnSetNode(node);

			m_oCondition = (OwnStuffCondition)node;

			lvwList.Items.Clear();
			for (int i = 0; i < m_oCondition.OwnThings.Count; i++)
			{
				OwnStuffCondition.OwnThing oThing = m_oCondition.OwnThings[i];
				AddThing(oThing);
			}

			LoadCitiesAndIndustries();
			SetControls();

			if (lvwList.Items.Count > 0)
			{
				lvwList.FocusedItem = lvwList.Items[0];
				lvwList.Items[0].Selected = true;
			}
		}

		private ListViewItem AddThing(OwnStuffCondition.OwnThing thing)
		{
			ListViewItem lvi = new ListViewItem(thing.City.Name);
			lvi.SubItems.Add(thing.IndustryType.Name);
			lvi.Tag = thing;

			lvwList.Items.Add(lvi);

			return lvi;
		}

		private void lvwList_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (lvwList.SelectedItems.Count > 0)
			{
				LoadCitiesAndIndustries();
				SetControls();
			}
		}

		private void LoadCitiesAndIndustries()
		{
			Program.SuspendPaint(this);
			
			ListViewItem lviSelected = SelectedItem;

			cboCity.Items.Clear();
			cboCity.Items.Add(ScenarioCity.AnyCity);
			foreach (ScenarioCity oCity in SelectedScenario.Cities.Values)
			{
				cboCity.Items.Add(oCity);
			}

			cboIndustry.Items.Clear();
			foreach (Industry oIndustry in SelectedScenario.Industries.Values)
			{
				cboIndustry.Items.Add(oIndustry);
			}
			
			Program.ResumePaint(this);
		}

		private void SetControls()
		{
			OwnStuffCondition.OwnThing oThing = SelectedOwnThing;
			if (oThing != null)
			{
				cboCity.SelectedItem = oThing.City;
				cboCity.Enabled = true;

				cboIndustry.SelectedItem = oThing.IndustryType;
				cboIndustry.Enabled = true;

				mnuAddNew.Enabled = (cboCity.Items.Count > 0);
				mnuDelete.Enabled = true;
			}
			else
			{
				cboCity.SelectedIndex = -1;
				cboCity.Enabled = false;

				cboIndustry.SelectedIndex = -1;
				cboIndustry.Enabled = false;

				mnuAddNew.Enabled = true;
				mnuDelete.Enabled = false;
			}
		}

		private void mnuAddNew_Click(object sender, EventArgs e)
		{
			AddNew();
		}

		private void AddNew()
		{
			OwnStuffCondition.OwnThing oThing = new OwnStuffCondition.OwnThing((ScenarioSettings)m_oCondition.RootSetting);

			foreach (ScenarioCity oCity in cboCity.Items)
			{
				if (oCity == cboCity.SelectedItem) continue;	//skip if it is current selection

				oThing.City = oCity;
				oThing.IndustryType = (Industry)cboIndustry.Items[0];

				m_oCondition.OwnThings.Add(oThing);
				ListViewItem lviNew = AddThing(oThing);

				lvwList.SelectedItems.Clear();
				lviNew.Selected = true;
				lvwList.FocusedItem = lviNew;

				break;
			}

			LoadCitiesAndIndustries();
			SetControls();
			SetNodeText();

			lvwList.Focus();
		}

		private void mnuDelete_Click(object sender, EventArgs e)
		{
			Delete();
		}

		private void Delete()
		{
			int iIndex = lvwList.SelectedIndices[0];

			m_oCondition.OwnThings.RemoveAt(iIndex);
			lvwList.Items.RemoveAt(iIndex);

			if (iIndex >= lvwList.Items.Count) iIndex--;
			if (iIndex >= 0)
			{
				lvwList.Items[iIndex].Selected = true;
				lvwList.FocusedItem = lvwList.Items[iIndex];
			}

			SetControls();
			SetNodeText();

			lvwList.Focus();
		}

		private void cboCity_SelectedIndexChanged(object sender, EventArgs e)
		{
			OwnStuffCondition.OwnThing oThing = SelectedOwnThing;
			if (oThing != null)
			{
				if (oThing.City != (ScenarioCity)cboCity.SelectedItem)
				{
					oThing.City = (ScenarioCity)cboCity.SelectedItem;
					SelectedItem.Text = oThing.City.Name;
				}
			}

			SetControls();
			SetNodeText();
		}

		private void cboIndustry_SelectedIndexChanged(object sender, EventArgs e)
		{
			OwnStuffCondition.OwnThing oThing = SelectedOwnThing;
			if (oThing != null)
			{
				if (oThing.IndustryType != (Industry)cboIndustry.SelectedItem)
				{
					oThing.IndustryType = (Industry)cboIndustry.SelectedItem;
					SelectedItem.SubItems[1].Text = oThing.IndustryType.Name;
				}
			}

			SetControls();
			SetNodeText();
		}

		protected override void OnResize(EventArgs e)
		{
			base.OnResize(e);

			if (lvwList != null)
			{
				lvwList.Columns[1].Width = -2;
			}
		}


		private OwnStuffCondition.OwnThing SelectedOwnThing
		{
			get
			{
				ListViewItem lvi = SelectedItem;
				if (lvi == null) return null;

				return (OwnStuffCondition.OwnThing)lvi.Tag;
			}
		}

		private ListViewItem SelectedItem
		{
			get { return (lvwList.SelectedItems.Count > 0) ? lvwList.SelectedItems[0] : null; }
		}


		private void lvwList_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.KeyCode)
			{
				case Keys.Insert:
					if (mnuAddNew.Enabled)
					{
						AddNew();
					}
					break;

				case Keys.Delete:
					if (mnuDelete.Enabled && (e.Modifiers == Keys.Shift))
					{
						Delete();
					}
					break;
			}
		}

		private void lvwList_KeyUp(object sender, KeyEventArgs e)
		{
			LoadCitiesAndIndustries();
			SetControls();
		}

		private void lvwList_MouseUp(object sender, MouseEventArgs e)
		{
			LoadCitiesAndIndustries();
			SetControls();
		}
	}
}
