using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Data;
using System.Globalization;
using System.Text;
using System.Windows.Forms;

using Railroads.ScenarioManager.Core;


namespace Railroads.ScenarioManager
{
	public partial class IndustryEditor : UserControl
	{
		private Scenario		m_oSelectedScenario;
		private bool			m_bOverridesResources;
		private bool			m_bIgnoreEvents = false;

		public IndustryEditor()
		{
			InitializeComponent();

			coh.Width = -2;
		}


		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public Scenario SelectedScenario 
		{
			get { return m_oSelectedScenario; }
			set { m_oSelectedScenario = value; LoadIndustries(); } 
		}


		private void LoadIndustries()
		{
			lvwIndustries.Items.Clear();

			if (m_oSelectedScenario != null)
			{
				foreach (ScenarioIndustry oIndustry in m_oSelectedScenario.Industries.Values)
				{
					AddIndustry(oIndustry);
				}
			}

			Program.SuspendPaint(this);

			LoadDetails();
			SetControls();

			if (lvwIndustries.Items.Count > 0)
			{
				lvwIndustries.FocusedItem = lvwIndustries.Items[0];
				lvwIndustries.Items[0].Selected = true;
			}

			Program.ResumePaint(this);

			LoadContextMenu();
		}

		private ListViewItem AddIndustry(ScenarioIndustry industry)
		{
			ListViewItem lvi = new ListViewItem(industry.Name);
			lvi.Tag = industry;

			lvwIndustries.Items.Add(lvi);

			return lvi;
		}

		private void LoadContextMenu()
		{
			Manager oManager = Manager.Instance;

			SortedList<string, Industry> oList = new SortedList<string, Industry>();
			foreach (Industry oIndustry in oManager.BaseIndustries.Values)
			{
				bool bFound = false;
				foreach (ListViewItem lvi in lvwIndustries.Items)
				{
					if (((ScenarioIndustry)lvi.Tag).Name == oIndustry.Name)
					{
						bFound = true;
						break;
					}
				}

				if (bFound) continue;

				oList.Add(oIndustry.Name, oIndustry);
			}

			mnuAddStockIndustry.DropDownItems.Clear();
			foreach (Industry oIndustry in oList.Values)
			{
				ToolStripItem mnu = mnuAddStockIndustry.DropDownItems.Add(oIndustry.Name);
				mnu.Tag = oIndustry;
				mnu.Click += new EventHandler(mnu_Click);
			}
		}

		private void mnu_Click(object sender, EventArgs e)
		{
			Industry oIndustry = ((ToolStripItem)sender).Tag as Industry;

			ScenarioIndustry oNewIndustry = new ScenarioIndustry(m_oSelectedScenario.ScenarioSettings);
			oIndustry.CopyToScenarioIndusty(oNewIndustry);

			m_oSelectedScenario.Industries.Add(oNewIndustry.Name, oNewIndustry);

			//Make sure the scenario has the good used by this industry
			foreach (Industry.Resource oResource in oNewIndustry.Resources)
			{
				((MainForm)ParentForm).CheckGood(oResource.Input, oResource.Output);
			}

			m_oSelectedScenario.IsDirty = true;

			ListViewItem lvi = AddIndustry(oNewIndustry);
			lvwIndustries.Sort();

			lvwIndustries.SelectedItems.Clear();
			lvwIndustries.FocusedItem = lvi;
			lvi.Selected = true;
		}


		private void mnuAddNew_Click(object sender, EventArgs e)
		{
			AddNew();
		}

		private void mnuRemoveIndustry_Click(object sender, EventArgs e)
		{
			Delete();
		}

		private void lvwIndustries_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (lvwIndustries.SelectedItems.Count > 0)
			{
				Program.SuspendPaint(this);

				LoadDetails();
				SetControls();

				Program.ResumePaint(this);
			}
		}

		private void lvwIndustries_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.KeyCode)
			{
				case Keys.Insert:
					if (mnuAddNew.Enabled)
					{
						AddNew();
					}
					break;

				case Keys.Delete:
					if (mnuRemoveIndustry.Enabled && (e.Modifiers == Keys.Shift))
					{
						Delete();
					}
					break;
			}
		}

		private void lvwIndustries_KeyUp(object sender, KeyEventArgs e)
		{
			SetControls();
		}

		private void lvwIndustries_MouseUp(object sender, MouseEventArgs e)
		{
			SetControls();
		}

		private void AddNew()
		{
			ScenarioIndustry oIndustry = new ScenarioIndustry(SelectedScenario.ScenarioSettings);
			oIndustry.Name = "New Industry";
			oIndustry.Cost = 500000;
			oIndustry.IsInCity = true;

			SelectedScenario.Industries.Add(oIndustry.Name, oIndustry);

			ListViewItem lvi = AddIndustry(oIndustry);

			lvwIndustries.FocusedItem = lvi;
			lvi.Selected = true;

			lvwIndustries.Focus();
		}

		private void Delete()
		{
			int iIndex = lvwIndustries.SelectedIndices[0];

			SelectedScenario.Industries.Remove(SelectedIndustry.Name);
			lvwIndustries.Items.RemoveAt(iIndex);

			if (iIndex >= lvwIndustries.Items.Count) iIndex--;
			if (iIndex >= 0)
			{
				lvwIndustries.Items[iIndex].Selected = true;
				lvwIndustries.FocusedItem = lvwIndustries.Items[iIndex];
			}

			SetControls();

			lvwIndustries.Focus();
		}

		private void SetControls()
		{
			m_bIgnoreEvents = true;

			lvwResources.Items.Clear();

			ScenarioIndustry oIndustry = SelectedIndustry;
			if (oIndustry != null)
			{
				txtIndustryName.Text = oIndustry.Name;
				txtIndustryName.Enabled = true;
				txtIndustryName.ForeColor = Color.Black;

				chkIndustryInCity.Checked = oIndustry.IsInCity;
				chkIndustryInCity.Enabled = true;

				txtIndustryCost.Text = oIndustry.Cost.ToString("N0");
				txtIndustryCost.Enabled = oIndustry.IsInCity;

				chkIndustryCostal.Checked = oIndustry.IsCostal;
				chkIndustryCostal.Enabled = true;

				chkIndustryMountainous.Checked = oIndustry.IsMountainous;
				chkIndustryMountainous.Enabled = true;

				txtScale.Text = oIndustry.Scale.ToString();
				txtScale.Enabled = true;

				txtModel.Text = oIndustry.Model;
				txtModel.Enabled = true;

				txtGrowKFM.Text = oIndustry.GrowKfm;
				txtGrowKFM.Enabled = true;

				txtAuctionImage.Text = oIndustry.AuctionImage;
				txtAuctionImage.Enabled = true;

				txtReportImage.Text = oIndustry.ReportImage;
				txtReportImage.Enabled = true;

				txtIcon.Text = oIndustry.IconFile;
				txtIcon.Enabled = true;

				LoadResources();
				lvwResources.Enabled = true;

				mnuAddNew.Enabled = true;
				mnuRemoveIndustry.Enabled = true;
			}
			else
			{
				txtIndustryName.Text = "";
				txtIndustryName.Enabled = false;

				chkIndustryInCity.Checked = false;
				chkIndustryInCity.Enabled = false;

				txtIndustryCost.Text = "";
				txtIndustryCost.Enabled = false;

				chkIndustryCostal.Checked = false;
				chkIndustryCostal.Enabled = false;

				chkIndustryMountainous.Checked = false;
				chkIndustryMountainous.Enabled = false;

				txtScale.Text = "";
				txtScale.Enabled = false;

				txtModel.Text = "";
				txtModel.Enabled = false;

				txtGrowKFM.Text = "";
				txtGrowKFM.Enabled = false;

				txtAuctionImage.Text = "";
				txtAuctionImage.Enabled = false;

				txtReportImage.Text = "";
				txtReportImage.Enabled = false;

				txtIcon.Text = "";
				txtIcon.Enabled = false;

				lvwResources.Enabled = false;

				cboInput.SelectedItem = null;
				cboInput.Enabled = false;

				cboOutput.SelectedItem = null;
				cboOutput.Enabled = false;

				txtRatio.Text = "";
				txtRatio.Enabled = false;

				mnuAddNew.Enabled = true;
				mnuRemoveIndustry.Enabled = false;
			}

			m_bIgnoreEvents = false;
		}

		private ScenarioIndustry SelectedIndustry
		{
			get
			{
				ListViewItem lvi = SelectedItem;
				if (lvi == null) return null;

				return (ScenarioIndustry)lvi.Tag;
			}
		}

		private ListViewItem SelectedItem
		{
			get { return (lvwIndustries.SelectedItems.Count > 0) ? lvwIndustries.SelectedItems[0] : null; }
		}


		private void LoadDetails()
		{
			cboInput.Items.Clear();
			cboOutput.Items.Clear();

			cboInput.Items.Add(ScenarioGood.NoGood);
			cboOutput.Items.Add(ScenarioGood.NoGood);

			if (SelectedScenario != null)
			{ 
				foreach (ScenarioGood oGood in SelectedScenario.Goods.Values)
				{
					cboInput.Items.Add(oGood);
					cboOutput.Items.Add(oGood);
				}
			}
		}

		private void LoadResources()
		{
			if (SelectedIndustry.Resources.Count > 0)
			{
				foreach (ScenarioIndustry.Resource oResource in SelectedIndustry.Resources)
				{
					AddResource(oResource);
				}

				m_bOverridesResources = true;
			}
			else 
			{
				//Otherwise use base industry resources
				Industry oBaseIndustry;
				if (Manager.Instance.BaseIndustries.TryGetValue(SelectedIndustry.Name, out oBaseIndustry))
				{
					foreach (Industry.Resource oResource in oBaseIndustry.Resources)
					{
						AddResource(oResource);
					}
				}

				m_bOverridesResources = false;

			}

			SetResourceListFont();

			if (lvwResources.Items.Count > 0)
			{
				lvwResources.FocusedItem = lvwResources.Items[0];
				lvwResources.Items[0].Selected = true;
			}
			else 
			{
				cboInput.SelectedItem = null;
				cboInput.Enabled = false;

				cboOutput.SelectedItem = null;
				cboOutput.Enabled = false;

				txtRatio.Text = "";
				txtRatio.Enabled = false;
			}
		}

		private ListViewItem AddResource(ScenarioIndustry.Resource resource)
		{
			ListViewItem lvi = new ListViewItem(resource.Input.Name);
			lvi.SubItems.Add(resource.Output.Name);
			lvi.SubItems.Add(resource.InputOutputRatio.ToString());

			lvi.Tag = resource;

			lvwResources.Items.Add(lvi);

			return lvi;
		}

		private void EnsureOverridesResources()
		{
			if (!m_bOverridesResources)
			{
				foreach (ListViewItem lvi in lvwResources.Items)
				{
					Industry.Resource oResource = lvi.Tag as Industry.Resource;

					if (oResource != null)
					{
						Industry.Resource oClone = (Industry.Resource)oResource.Clone();
						oClone.SetRootSetting(m_oSelectedScenario.ScenarioSettings);

						SelectedIndustry.Resources.Add(oClone);

						lvi.Tag = oClone;
					}
				}

				m_oSelectedScenario.IsDirty = true;
				m_bOverridesResources = true;
			}

			SetResourceListFont();
		}

		private void SetResourceListFont()
		{
			lvwResources.Font = m_bOverridesResources ? this.Font : new Font(this.Font, FontStyle.Italic);
		}

		private void txtIndustryName_TextChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;

			string sNewName = txtIndustryName.Text.Trim();
			if (sNewName == "") return;

			bool bError = false;
			foreach (ScenarioIndustry oIndustry in SelectedScenario.Industries.Values)
			{
				if (oIndustry == SelectedIndustry) continue;

				if (oIndustry.Name == sNewName)
				{
					bError = true;
					break;
				}
			}

			if (bError)
			{
				txtIndustryName.ForeColor = Color.Red;
			}
			else 
			{
				txtIndustryName.ForeColor = Color.Black;

				SelectedScenario.Industries.Remove(SelectedIndustry.Name);
				SelectedIndustry.Name = sNewName;
				SelectedScenario.Industries.Add(sNewName, SelectedIndustry);
				SelectedItem.Text = sNewName;
			}
		}

		private void txtIndustryCost_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (e.KeyChar >= ' ')
			{
				if (((e.KeyChar < '0') || (e.KeyChar > '9')) && (e.KeyChar != '.') && (e.KeyChar != ','))
				{
					e.Handled = true;
				}
			}
		}

		private void txtIndustryCost_Leave(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;

			SetCost();
		}

		private void txtIndustryCost_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Enter)
			{
				SetCost();
			}
		}

		private void chkIndustryInCity_CheckedChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;

			if (SelectedIndustry != null)
			{
				txtIndustryCost.Enabled = chkIndustryInCity.Checked;

				SelectedIndustry.IsInCity = chkIndustryInCity.Checked;
				SetCost();
			}
		}

		private void SetCost()
		{
			if (SelectedIndustry != null)
			{
				int nCost = -1;
				if (int.TryParse(txtIndustryCost.Text, NumberStyles.Number, CultureInfo.CurrentCulture, out nCost))
				{
					txtIndustryCost.Text = nCost.ToString("N0");
				}

				SelectedIndustry.Cost = nCost;
			}
		}


		private void txtScale_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (e.KeyChar >= ' ')
			{
				if (((e.KeyChar < '0') || (e.KeyChar > '9')) && (e.KeyChar != '.') && (e.KeyChar != ','))
				{
					e.Handled = true;
				}
			}
		}

		private void txtScale_Leave(object sender, EventArgs e)
		{
			SetScale();
		}

		private void txtScale_KeyDown(object sender, KeyEventArgs e)
		{
			SetScale();
		}

		private void SetScale()
		{
			if (SelectedIndustry != null)
			{
				float fScale = -1;
				if (float.TryParse(txtScale.Text, NumberStyles.Number, CultureInfo.CurrentCulture, out fScale))
				{
					txtScale.Text = fScale.ToString();
				}

				SelectedIndustry.Scale = fScale;
			}
		}


		private void chkIndustryCostal_CheckedChanged(object sender, EventArgs e)
		{
			if (SelectedIndustry != null)
			{
				SelectedIndustry.IsCostal = chkIndustryCostal.Checked;
			}
		}

		private void chkIndustryMountainous_CheckedChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;

			if (SelectedIndustry != null)
			{
				SelectedIndustry.IsMountainous = chkIndustryMountainous.Checked;
			}
		}


		private void lvwResources_SelectedIndexChanged(object sender, EventArgs e)
		{
			SetControls2();
		}

		private void SetControls2()
		{
			m_bIgnoreEvents = true;

			ScenarioIndustry.Resource oResource = SelectedResource;
			if (oResource != null)
			{
				SelectResource(cboInput, oResource.Input);
				cboInput.Enabled = true;

				SelectResource(cboOutput, oResource.Output);
				cboOutput.Enabled = true;

				txtRatio.Text = oResource.InputOutputRatio.ToString();
				txtRatio.Enabled = true;

				mnuAddResource.Enabled = true;
				mnuRemoveResource.Enabled = true;
			}
			else 
			{
				cboInput.SelectedItem = null;
				cboInput.Enabled = false;

				cboOutput.SelectedItem = null;
				cboOutput.Enabled = false;

				txtRatio.Text = "";
				txtRatio.Enabled = false;

				mnuAddResource.Enabled = lvwResources.Enabled;
				mnuRemoveResource.Enabled = false;
			}

			m_bIgnoreEvents = false;
		}

		internal static void SelectResource(ComboBox comboBox, Good good)
		{
			foreach (ScenarioGood oGood in comboBox.Items)
			{
				if (oGood.Name == good.Name)
				{
					comboBox.SelectedItem = oGood;
					return;
				}
			}
			comboBox.SelectedItem = null;
		}


		private void mnuAddResource_Click(object sender, EventArgs e)
		{
			AddResource();
		}

		private void mnuRemoveResource_Click(object sender, EventArgs e)
		{
			RemoveResource();
		}

		private void lvwResources_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.KeyCode)
			{
				case Keys.Insert:
					if (mnuAddResource.Enabled)
					{
						AddResource();
					}
					break;

				case Keys.Delete:
					if (mnuRemoveResource.Enabled && (e.Modifiers == Keys.Shift))
					{
						RemoveResource();
					}
					break;
			}
		}

		private void lvwResources_KeyUp(object sender, KeyEventArgs e)
		{
			SetControls2();
		}

		private void lvwResources_MouseUp(object sender, MouseEventArgs e)
		{
			SetControls2();
		}

		private void AddResource()
		{
			EnsureOverridesResources();

			ScenarioIndustry.Resource oResource = new ScenarioIndustry.Resource(m_oSelectedScenario.ScenarioSettings);
			SelectedIndustry.Resources.Add(oResource);

			oResource.Input = ScenarioGood.NoGood;
			oResource.Output = ScenarioGood.NoGood;
			oResource.InputOutputRatio = 1;

			ListViewItem lvi = AddResource(oResource);

			lvwResources.FocusedItem = lvi;
			lvi.Selected = true;

			lvwResources.Focus();
		}

		private void RemoveResource()
		{
			EnsureOverridesResources();

			int iIndex = lvwResources.SelectedIndices[0];

			SelectedIndustry.Resources.RemoveAt(iIndex);
			lvwResources.Items.RemoveAt(iIndex);

			if (iIndex >= lvwResources.Items.Count) iIndex--;
			if (iIndex >= 0)
			{
				lvwResources.Items[iIndex].Selected = true;
				lvwResources.FocusedItem = lvwResources.Items[iIndex];
			}

			SetControls2();

			lvwResources.Focus();
		}

		private void cboInput_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;
			if (SelectedResource == null) return;

			EnsureOverridesResources();
			SelectedResource.Input = (Good)cboInput.SelectedItem;
			SelectedItem2.Text = SelectedResource.Input.Name;
		}

		private void cboOutput_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;
			if (SelectedResource == null) return;

			EnsureOverridesResources();
			SelectedResource.Output = (Good)cboOutput.SelectedItem;
			SelectedItem2.SubItems[1].Text = SelectedResource.Output.Name;
		}

		private void txtRatio_TextChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;
			if (SelectedResource == null) return;

			if (txtRatio.Text != "")
			{
				float fRatio;
				if (float.TryParse(txtRatio.Text, NumberStyles.Number, CultureInfo.CurrentCulture, out fRatio))
				{
					EnsureOverridesResources();

					SelectedResource.InputOutputRatio = fRatio;
					SelectedItem2.SubItems[2].Text = fRatio.ToString();
				}
			}
		}

		private void txtRatio_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (e.KeyChar >= ' ')
			{
				if (((e.KeyChar < '0') || (e.KeyChar > '9')) && (e.KeyChar != '.') && (e.KeyChar != ','))
				{
					e.Handled = true;
				}
			}
		}

		private ScenarioIndustry.Resource SelectedResource
		{
			get
			{
				ListViewItem lvi = SelectedItem2;
				if (lvi == null) return null;

				return (ScenarioIndustry.Resource)lvi.Tag;
			}
		}

		private ListViewItem SelectedItem2
		{
			get { return (lvwResources.SelectedItems.Count > 0) ? lvwResources.SelectedItems[0] : null; }
		}




		private void txtModel_TextChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;

			SelectedIndustry.Model = txtModel.Text;
		}

		private void txtGrowKFM_TextChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;

			SelectedIndustry.GrowKfm = txtGrowKFM.Text;
		}

		private void txtAuctionImage_TextChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;

			SelectedIndustry.AuctionImage = txtAuctionImage.Text;
		}

		private void txtReportImage_TextChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;

			SelectedIndustry.ReportImage = txtReportImage.Text;
		}

		private void txtIcon_TextChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;

			SelectedIndustry.IconFile = txtIcon.Text;
		}

		private void txtScale_TextChanged(object sender, EventArgs e)
		{

		}

		private void txtIndustryCost_TextChanged(object sender, EventArgs e)
		{

		}
	}
}
