using System;
using System.Globalization;
using System.IO;
using System.Runtime.InteropServices;
using System.Windows.Forms;

using Railroads.ScenarioManager.Core;


namespace Railroads.ScenarioManager
{
	static class Program
	{
		private static MainForm _mainForm;

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main()
		{
			// setup a global exception handler
			AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(OnUnhandledException);

			Application.EnableVisualStyles();
			Application.SetCompatibleTextRenderingDefault(false);

			//Set the current Culture to en-US as most of the xml files are stored this way (the exeception being the 
			// locale specific file).
			Application.CurrentCulture = new CultureInfo("en-US");
			
			
			if (!ValidateOptions())
			{
				using (OptionsForm form = new OptionsForm())
				{
					if (form.ShowDialog() != DialogResult.OK)
					{
						MessageBox.Show("You must set all the options to be able to use the Scenario Manager.");

						return;
					}
				}
			}

			try
			{
				Manager.Instance.Load();

				_mainForm = new MainForm();

				Application.Run(_mainForm);

				Manager.Instance.Save();
			}
			catch (Exception ex)
			{
				OnUnhandledException(null, new UnhandledExceptionEventArgs(ex, true));
			}
		}

		private static bool ValidateOptions()
		{
			bool bAllValid = true;

			if (string.IsNullOrEmpty(ManagerApplication.AppSettings.RailroadsDirectory) 
				|| !Directory.Exists(ManagerApplication.AppSettings.RailroadsDirectory))
			{
				bAllValid = false;

				if (Directory.Exists(Locations.Default_GameFolder))
				{
					ManagerApplication.AppSettings.RailroadsDirectory = Locations.Default_GameFolder;
				}
			}

			if (string.IsNullOrEmpty(ManagerApplication.AppSettings.MyRailroadsDirectory)
				|| !Directory.Exists(ManagerApplication.AppSettings.MyRailroadsDirectory))
			{
				bAllValid = false;

				if (Directory.Exists(Locations.Default_MyGamesRailroadsFolder))
				{
					ManagerApplication.AppSettings.MyRailroadsDirectory = Locations.Default_MyGamesRailroadsFolder;
				}
			}
			
			if (string.IsNullOrEmpty(ManagerApplication.AppSettings.WorkingDirectory)
				|| !Directory.Exists(ManagerApplication.AppSettings.WorkingDirectory))
			{
				if (Directory.Exists(Locations.Default_WorkingDirectory))
				{
					ManagerApplication.AppSettings.WorkingDirectory = Locations.Default_WorkingDirectory;
				}
				else
				{
					try
					{
						Directory.CreateDirectory(Locations.Default_WorkingDirectory);
						ManagerApplication.AppSettings.WorkingDirectory = Locations.Default_WorkingDirectory;
					}
					catch (Exception)
					{
					}
				}

				bAllValid = false;
			}

			return bAllValid;
		}

		private static void OnUnhandledException(object sender, UnhandledExceptionEventArgs e)
		{
			Exception ex = e.ExceptionObject as Exception;

			if (ex == null)
			{
				ex = new Exception("Could not load exception.");
			}

			using (UnhandledExceptionForm form = new UnhandledExceptionForm())
			{
				form.Exception = ex;

				form.ShowDialog();
			}

			if (_mainForm != null)
			{
				_mainForm.Close();
			}

			Application.Exit();
		}


		/// <summary>
		/// Suspends PAINT processing for given control
		/// </summary>
		public static void SuspendPaint(Control control)
		{
			SendMessage(control.Handle, MSG.WM_SETREDRAW, 0, 0);
		}

		/// <summary>
		/// Resumes PAINT processing for given control
		/// </summary>
		public static void ResumePaint(Control control)
		{
			SendMessage(control.Handle, MSG.WM_SETREDRAW, 1, 0);
			control.Invalidate(true);
			control.Update();
		}

		/// <summary>
		/// Windows API SendMessage function
		/// </summary>
		[DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
		public static extern int SendMessage(IntPtr hWnd, MSG msg, int wParam, int lParam);
	}


	[Flags]
	public enum MSG
	{
		WM_SETREDRAW = 0x000B,
		WM_GETTEXTLENGTH = 0x000E,
		WM_PAINT = 0x000F,
		WM_ERASEBKGND = 0x14,
		WM_PRINT = 0x0317,

		EM_CUT = 0x300,
		EM_COPY = 0x301,
		EM_PASTE = 0x302,
		EM_CLEAR = 0x303,
		EM_UNDO = 0x304,

		WM_USER = 0x400,

		WM_NCCALCSIZE = 0x0083,
		WM_NCPAINT = 0x0085,

		WM_HSCROLL = 0x0114,
		WM_VSCROLL = 0x0115,

		LVM_FIRST = 0x1000,
		LVM_SCROLL = LVM_FIRST + 20,
		LVM_GETHEADER = LVM_FIRST + 31,
		LVM_SETCOLUMN = LVM_FIRST + 0x1A,
		LVM_SETCOLUMNORDERARRAY = LVM_FIRST + 58,
		LVM_GETCOLUMNORDERARRAY = LVM_FIRST + 59,

		HDM_SETIMAGELIST = 0x1208,

		EM_CANUNDO = 0xC6,
		EM_CHARFROMPOS = 0x00D7,
		EM_SETRECT = 0x00B3,
		EM_CANPASTE = WM_USER + 50,
		EM_CANREDO = WM_USER + 85,
		EM_REDO = WM_USER + 84,
		EM_GETTEXTEX = WM_USER + 94,
		EM_GETTEXTLENGTHEX = WM_USER + 95,
	}
}