using System;
using System.Collections.Generic;
using System.IO;
using System.Security.Permissions;
using ICSharpCode.SharpZipLib.Checksums;
using ICSharpCode.SharpZipLib.Zip;

namespace Railroads.ScenarioManager.Core
{
	public static class CompressionUtility
	{
		private const int WRITE_FILE_BUFFER_SIZE = 4096;


		public static void CompressFile(string file, string outputFile)
		{
			FileList oFileList = new FileList(file);
			CompressFiles(new FileList[] { oFileList }, outputFile);
		}


		public static void CompressFiles(FileList fileList, string outputFile)
		{
			CompressFiles(new FileList[] { fileList }, outputFile);
		}


		/// <summary>
		/// Compresses the files in the FileLists and writes the resulting zip file to the specified location.
		/// </summary>
		/// <param name="files">Files to add to Zip.</param>
		/// <param name="outputFile">Location for the resulting zip file.</param>
		/// <remarks>
		/// Based on the CreateZipFile sample included with the SharpZipLib source.
		/// </remarks>
		/// <see cref="http://www.icsharpcode.net/OpenSource/SharpZipLib/"/>
		public static void CompressFiles(ICollection<FileList> files, string outputFile)
		{
			//Make sure there is at least one file list defined
			if (files == null || files.Count == 0)
			{
				return;
			}

			//Make sure we can write to the output file
			new FileIOPermission(FileIOPermissionAccess.Write, Path.GetFullPath(outputFile)).Demand();
			

			using (ZipOutputStream oZipStream = new ZipOutputStream(File.Create(outputFile)))
			{
				foreach (FileList oFileList in files)
				{
					string sDest = oFileList.Destination;

					foreach (string sFileName in oFileList.GetFiles())
					{
						//Make sure we have read permission to this file
						new FileIOPermission(FileIOPermissionAccess.Read, Path.GetFullPath(sFileName)).Demand();

						//Read the file into a buffer and setup the ZipEntry for this file
						ZipEntry oZipEntry = null;
						Crc32 oCrc = new Crc32();
						byte[] aBuffer = null;

						using (FileStream oFileStream = File.OpenRead(sFileName))
						{
							aBuffer = new byte[oFileStream.Length];

							oFileStream.Read(aBuffer, 0, aBuffer.Length);

							oZipEntry = new ZipEntry(sDest + Path.GetFileName(sFileName));
							oZipEntry.DateTime = DateTime.Now;
							oZipEntry.Size = oFileStream.Length;

							oFileStream.Close();
						}

						//Calculate the CRC for this file
						oCrc.Reset();
						oCrc.Update(aBuffer);

						oZipEntry.Crc = oCrc.Value;

						oZipStream.PutNextEntry(oZipEntry);

						//Write buffer to zip stream
						oZipStream.Write(aBuffer, 0, aBuffer.Length);
					}
				}

				oZipStream.Finish();
				oZipStream.Close();
			}
		}


		/// <summary>
		/// Decompresses the specified zip file into the specified destination folder.
		/// </summary>
		/// <param name="file">Zip file to decompress.</param>
		/// <param name="destination">Folder to put the decompressed files into.</param>
		/// /// <remarks>
		/// Based on the UnzipFile sample included with the SharpZipLib source.
		/// </remarks>
		/// <see cref="http://www.icsharpcode.net/OpenSource/SharpZipLib/"/>
		/// <returns>List of files that were decompressed.</returns>
		public static string[] DecompressFile(string file, string destination)
		{
			destination = Utility.AddTrailingSlash(destination);

			CheckDecompressParameters(file, destination);

			List<string> oDecompressedFiles = new List<string>();

			using (ZipInputStream zipStream = new ZipInputStream(File.OpenRead(file)))
			{
				ZipEntry oZipEntry = zipStream.GetNextEntry();
				while (oZipEntry != null)
				{
					string sDir = destination + Path.GetDirectoryName(oZipEntry.Name);					

					//Make sure directory exists
					if (!Directory.Exists(sDir))
					{
						Directory.CreateDirectory(sDir);
					}

					//If this entry has a file then decompress it to the file
					if (Path.GetFileName(oZipEntry.Name) != String.Empty)
					{
						if (!string.IsNullOrEmpty(sDir))
						{
							sDir = Utility.AddTrailingSlash(sDir);
						}

						string sOutputFile = sDir + Path.GetFileName(oZipEntry.Name);
						oDecompressedFiles.Add(sOutputFile);

						using (FileStream oStreamWriter = File.Create(sOutputFile))
						{
							if (oZipEntry.Size > 0)
							{
								int nSize = WRITE_FILE_BUFFER_SIZE;
								byte[] aData = new byte[nSize];

								while (true)
								{
									nSize = zipStream.Read(aData, 0, aData.Length);
									if (nSize > 0)
									{
										oStreamWriter.Write(aData, 0, nSize);
									}
									else
									{
										break;
									}
								}
							}

							oStreamWriter.Close();
						}
					}
					else
					{
						oDecompressedFiles.Add(sDir);
					}

					oZipEntry = zipStream.GetNextEntry();
				}

				zipStream.Close();
			}

			return oDecompressedFiles.ToArray();
		}


		private static void CheckDecompressParameters(string file, string destination)
		{
			if (string.IsNullOrEmpty(file))
			{
				throw new ArgumentNullException("File");
			}

			if (string.IsNullOrEmpty(destination))
			{
				throw new ArgumentNullException("Destination");
			}

			if (!File.Exists(file))
			{
				throw new FileNotFoundException("The file '" + file + "' could not be found.");
			}

			new FileIOPermission(FileIOPermissionAccess.Read, Path.GetFullPath(file)).Demand();

			if (!Directory.Exists(destination))
			{
				throw new DirectoryNotFoundException("The directory '" + destination + "' could not be found.");
			}

			new FileIOPermission(FileIOPermissionAccess.Write, Path.GetFullPath(destination)).Demand();
		}


		public static bool IsZipFile(string file)
		{
			try
			{
				new ZipFile(file);

				return true;
			}
			catch
			{
				return false;
			}
		}
	}
}
