using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.IO;
using System.Text;
using System.Xml;

namespace Railroads.ScenarioManager.Core
{
	public static class EntityLoader
	{
		internal static void LoadBaseEntities(Manager manager, ApplicationSettings settings)
		{
			manager.BaseGoods = new Dictionary<string, Good>();
			List<Good> oGoods = LoadBaseEntity<Good>(settings.RailroadsDirectory + Constants.ASSETS + @"\Xml\RRT_Goods.xml", "/RRTGoods/Good");
			foreach (Good oGood in oGoods)
			{
				manager.BaseGoods.Add(oGood.Name, oGood);
			}

			manager.BaseCityTypes = new Dictionary<string, CityType>();
			List<CityType> oCityTypes = LoadBaseEntity<CityType>(settings.RailroadsDirectory + Constants.ASSETS + @"\Xml\RRT_Cities.xml", "/RRTCities/RRTCityTypes/CityType");
			foreach (CityType oCityType in oCityTypes)
			{
				manager.BaseCityTypes.Add(oCityType.Name, oCityType);
			}
			
			manager.BaseIndustries = new Dictionary<string, Industry>();
			List<Industry> oIndustries = LoadBaseEntity<Industry>(settings.RailroadsDirectory + Constants.ASSETS + @"\Xml\RRT_Industries.xml", "/RRTIndustries/Industries/RRTIndustry");
			foreach (Industry oIndustry in oIndustries)
			{
				manager.BaseIndustries.Add(oIndustry.Name, oIndustry);
			}
			
			manager.BaseDepots = new Dictionary<string, Depot>();
			List<Depot> oDepots = LoadBaseEntity<Depot>(settings.RailroadsDirectory + Constants.ASSETS + @"\Xml\RRT_Depots.xml", "/RRTDepots/Depot");
			foreach (Depot oDepot in oDepots)
			{
				manager.BaseDepots.Add(oDepot.Name, oDepot);
			}

			manager.BaseTrainCars = new Dictionary<string, TrainGoodCar>();
			List<TrainGoodCar> oCars = LoadBaseEntity<TrainGoodCar>(settings.RailroadsDirectory + Constants.ASSETS + @"\Xml\RRT_TrainCars.xml", "/RRTTrainCars/RRTGoodCars/TrainCar");
			foreach (TrainGoodCar oCar in oCars)
			{
				manager.BaseTrainCars.Add(oCar.Name, oCar);

				//Add good car to good's car list
				Good oTempGood;
				if (manager.BaseGoods.TryGetValue(oCar.Good.Name, out oTempGood))
				{
					oTempGood.TrainCars.Add(oCar);
				}
			}

			manager.Sounds3d = new Dictionary<string, Sound3d>();
			LoadBaseSounds(settings.RailroadsDirectory + Constants.ASSETS + @"\Xml\Audio\Audio3DScripts.xml", manager);
		}


		private static List<T> LoadBaseEntity<T>(string file, string xPath) where T : XmlSetting, new()
		{
			if (!File.Exists(file))
			{
				throw new ArgumentException("Could not find entity file at '" + file + "'");
			}

			XmlDocument oDoc = new XmlDocument();
			using (StreamReader oReader = new StreamReader(file, Encoding.UTF8))
			{
				oDoc.Load(oReader);
				oReader.Close();
			}

			List<T> results = new List<T>();
			XmlNodeList oNodeList = oDoc.SelectNodes(xPath);
			if ((oNodeList != null) || (oNodeList.Count > 0))
			{
				foreach (XmlNode oNode in oNodeList)
				{
					T entity = new T();
					entity.FromXml(null, oNode);
					results.Add(entity);
				}
			}

			return results;
		}


		private static void LoadBaseSounds(string file, Manager manager)
		{
			if (!File.Exists(file))
			{
				throw new ArgumentException("Could not find entity file at '" + file + "'");
			}

			XmlDocument oDoc = new XmlDocument();

			using (StreamReader oReader = new StreamReader(file, Encoding.UTF8))
			{
				using (XmlTextReader oXmlReader = new XmlTextReader(oReader))
				{
					oXmlReader.Namespaces = false;

					oDoc.Load(oXmlReader);

					oXmlReader.Close();
					oReader.Close();
				}
			}

			XmlNodeList oNodeList = oDoc.SelectNodes("/Script3DSounds/Script3DSound[SoundType='GAME_MUSIC']");
			if ((oNodeList != null) || (oNodeList.Count > 0))
			{
				foreach (XmlNode oNode in oNodeList)
				{
					Sound3d oSound = new Sound3d();
					oSound.FromXml(null, oNode);
					manager.Sounds3d.Add(oSound.ScriptID, oSound);
				}
			}
		}


		public static void LoadEntities(Scenario scenario, ScenarioFiles files)
		{
			scenario.Names = new Dictionary<string, City>();
			List<City> oNames = LoadEntity<City>(scenario, files.Names, "/Names/RRTCities/TextKey");
			if (oNames != null)
			{
				foreach (City oName in oNames)
				{
					if (scenario.Names.ContainsKey(oName.Tag))
					{
						scenario.Problems.Add(string.Format("City tag '{0}' defined twice in '{1}'.", oName.Tag, files.Names));
					}
					else
					{
						scenario.Names.Add(oName.Tag, oName);
					}
				}
			}


			scenario.BuildingSpacing = LoadIntValue(scenario, files.Cities, "/RRTCities/GridSize", "BuildingSpacing");


			scenario.Cities = new Dictionary<string, ScenarioCity>();
			List<ScenarioCity> oCities = LoadEntity<ScenarioCity>(scenario, files.Cities, "/RRTCities/City");
			foreach (ScenarioCity oCity in oCities)
			{
				if (scenario.Cities.ContainsKey(oCity.Tag))
				{
					scenario.Problems.Add(string.Format("City '{0}' defined twice in '{1}'.", oCity.Tag, files.Cities));
				}
				else 
				{
					scenario.Cities.Add(oCity.Tag, oCity);
				}
			}


			scenario.Goods = new Dictionary<string, ScenarioGood>();
			List<ScenarioGood> oGoods = LoadEntity<ScenarioGood>(scenario, files.Goods, "/RRTGoods/Good");
			foreach (ScenarioGood oGood in oGoods)
			{
				if (scenario.Goods.ContainsKey(oGood.Name))
				{
					scenario.Problems.Add(string.Format("Good '{0}' defined twice in '{1}'.", oGood.Name, files.Goods));
				}
				else
				{
					scenario.Goods.Add(oGood.Name, oGood);
				}
			}


			scenario.CityTypes = new Dictionary<string, ScenarioCityType>();
			List<ScenarioCityType> oCityTypes = LoadEntity<ScenarioCityType>(scenario, files.Cities, "RRTCities/RRTCityTypes/CityType");
			foreach (ScenarioCityType oCityType in oCityTypes)
			{
				if (scenario.CityTypes.ContainsKey(oCityType.Name))
				{
					scenario.Problems.Add(string.Format("City type '{0}' defined twice in '{1}'.", oCityType.Name, files.Cities));
				}
				else 
				{
					scenario.CityTypes.Add(oCityType.Name, oCityType);
				}
			}


			scenario.Models = new Dictionary<string, Model>();
			List<Model> oModels = LoadEntity<Model>(scenario, files.Cities, "RRTCities/RRTModels/Model");
			foreach (Model oModel in oModels)
			{
				if (scenario.Models.ContainsKey(oModel.NifName))
				{
					scenario.Problems.Add(string.Format("Model '{0}' defined twice in '{1}'.", oModel.NifName, files.Cities));
				}
				else
				{
					scenario.Models.Add(oModel.NifName, oModel);
				}
			}


			scenario.Industries = new Dictionary<string, ScenarioIndustry>();
			List<ScenarioIndustry> oIndustries = LoadEntity<ScenarioIndustry>(scenario, files.Industries, "/RRTIndustries/Industries/RRTIndustry");
			foreach (ScenarioIndustry oIndustry in oIndustries)
			{
				if (scenario.Industries.ContainsKey(oIndustry.Name))
				{
					scenario.Problems.Add(string.Format("Industry '{0}' defined twice in '{1}'.", oIndustry.Name, files.Industries));
				}
				else 
				{
					scenario.Industries.Add(oIndustry.Name, oIndustry);
				}
			}


			scenario.Depots = new Dictionary<string, ScenarioDepot>();
			List<ScenarioDepot> oDepots = LoadEntity<ScenarioDepot>(scenario, files.Depots, "/RRTDepots/Depot");
			foreach (ScenarioDepot oDepot in oDepots)
			{
				if (scenario.Depots.ContainsKey(oDepot.Name))
				{
					scenario.Problems.Add(string.Format("Depot '{0}' defined twice in '{1}'.", oDepot.Name, files.Depots));
				}
				else 
				{
					scenario.Depots.Add(oDepot.Name, oDepot);
				}
			}


			scenario.Trains = new Dictionary<string, ScenarioTrain>();
			List<ScenarioTrain> oTrains = LoadEntity<ScenarioTrain>(scenario, files.Trains, "/RRTTrains/Train");
			foreach (ScenarioTrain oTrain in oTrains)
			{
				if (scenario.Trains.ContainsKey(oTrain.Tag))
				{
					scenario.Problems.Add(string.Format("Train '{0}' defined twice in '{1}'.", oTrain.Tag, files.Trains));
				}
				else 
				{
					scenario.Trains.Add(oTrain.Tag, oTrain);
				}
			}


			scenario.TrainGoodCars = new Dictionary<string, ScenarioTrainGoodCar>();
			List<ScenarioTrainGoodCar> oGoodCars = LoadEntity<ScenarioTrainGoodCar>(scenario, files.TrainCars, "/RRTTrainCars/RRTGoodCars/TrainCar");
			foreach (ScenarioTrainGoodCar oGoodCar in oGoodCars)
			{
				if (scenario.TrainGoodCars.ContainsKey(oGoodCar.Name))
				{
					scenario.Problems.Add(string.Format("Train Good Car '{0}' defined twice in '{1}'.", oGoodCar.Name, files.TrainCars));
				}
				else 
				{
					scenario.TrainGoodCars.Add(oGoodCar.Name, oGoodCar);

					//New for RR 1.1
					//Check to see if specified scenario good does not have this scenario car
					if (!string.IsNullOrEmpty(oGoodCar.GoodName))
					{
						ScenarioGood oTempGood;
						if (scenario.Goods.TryGetValue(oGoodCar.GoodName, out oTempGood))
						{
							if (oTempGood.TrainCars.Count == 0)
							{
								oTempGood.TrainCars.Add(oGoodCar);
							}
						}
					}
				}
			}


			scenario.TrainTenderCars = new Dictionary<string, ScenarioTrainTenderCar>();
			List<ScenarioTrainTenderCar> oTenderCars = LoadEntity<ScenarioTrainTenderCar>(scenario, files.TrainCars, "/RRTTrainCars/RRTTenderCars/TenderCar");
			foreach (ScenarioTrainTenderCar oTenderCar in oTenderCars)
			{
				scenario.TrainTenderCars.Add(oTenderCar.Name, oTenderCar);
			}
		}


		private static List<T> LoadEntity<T>(Scenario scenario, string path, string xPath) where T : XmlSetting, new()
		{
			if (string.IsNullOrEmpty(path))
			{
				return null;
			}

			XmlDocument oDoc = new XmlDocument();

			try
			{
				using (StreamReader oReader = new StreamReader(path, Encoding.UTF8))
				{
					oDoc.Load(oReader);
					oReader.Close();
				}
			}
			catch (Exception ex)
			{
				throw new Exception("Error while loading '" + path + "'", ex);				
			}

			XmlNodeList oNodes = oDoc.SelectNodes(xPath);
			List<T> oEntities = new List<T>();
			if ((oNodes != null) && (oNodes.Count > 0))
			{
				foreach (XmlNode oNode in oNodes)
				{
					T oEntity = new T();
					oEntity.RootSetting = scenario.ScenarioSettings;
					oEntity.FromXml(scenario, oNode);

					oEntities.Add(oEntity);
				}
			}

			return oEntities;
		}


		private static int LoadIntValue(Scenario scenario, string path, string xPath, string elementName)
		{
			XmlDocument oDoc = new XmlDocument();
			try
			{
				using (StreamReader oReader = new StreamReader(path, Encoding.UTF8))
				{
					oDoc.Load(oReader);
					oReader.Close();
				}
			}
			catch (Exception ex)
			{
				throw new Exception("Error while loading '" + path + "'", ex);
			}

			XmlNodeList oNodes = oDoc.SelectNodes(xPath);
			if ((oNodes != null) && (oNodes.Count > 0))
			{
				foreach (XmlNode oNode in oNodes)
				{
					return Utility.GetChildElementValue<int>(oNode, elementName);
				}
			}

			return 0;
		}
	}
}
