using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Xml;

namespace Railroads.ScenarioManager.Core
{
	public class Manager
	{
		private const string TAGGED_ITEMS_FILE = "Names_";
		private const string IMPORT_EXPORT_NAMES_FILE = "Names.xml";

		private static readonly Manager				ms_oInstance = new Manager();

		private Dictionary<string, Scenario>		m_oCoreScenarios;
		private Dictionary<string, Scenario>		m_oUserScenarios;
		private Dictionary<string, Scenario>		m_oLocalScenarios;
		private Dictionary<string, CityType>		m_oBaseCityTypes;
		private Dictionary<string, Good>			m_oBaseGoods;
		private Dictionary<string, Industry>		m_oBaseIndustries;
		private Dictionary<string, Depot>			m_oBaseDepots;
		private Dictionary<string, Train>			m_oBaseTrains;
		private Dictionary<string, TrainGoodCar>	m_oBaseTrainCars;
		private Dictionary<string, City>			m_oBaseCities;
		private Dictionary<string, Sound3d>			m_oSounds3D;

		private Dictionary<string, Exception>		m_oScenarioErrors = new Dictionary<string, Exception>();

		private bool								m_bNamesUpdated;
		private string[]							m_aCommonLocations;



		static Manager()
		{
		}

		private Manager()
		{
		}


		public static Manager Instance { get { return ms_oInstance; } }


		public Dictionary<string, Scenario> CoreScenarios { get { return m_oCoreScenarios; } }
		public Dictionary<string, Scenario> UserScenarios { get { return m_oUserScenarios; } set { m_oUserScenarios = value; } }
		public Dictionary<string, Scenario> LocalScenarios { get { return m_oLocalScenarios; } }

		public Dictionary<string, CityType> BaseCityTypes { get { return m_oBaseCityTypes; } set { m_oBaseCityTypes = value; } }
		public Dictionary<string, Good> BaseGoods { get { return m_oBaseGoods; } set { m_oBaseGoods = value; } }
		public Dictionary<string, Industry> BaseIndustries { get { return m_oBaseIndustries; } set { m_oBaseIndustries = value; } }
		public Dictionary<string, Depot> BaseDepots { get { return m_oBaseDepots; } set { m_oBaseDepots = value; } }
		public Dictionary<string, Train> BaseTrains { get { return m_oBaseTrains; } set { m_oBaseTrains = value; } }
		public Dictionary<string, TrainGoodCar> BaseTrainCars { get { return m_oBaseTrainCars; } set { m_oBaseTrainCars = value; } }
		public Dictionary<string, City> BaseCities { get { return m_oBaseCities; } set { m_oBaseCities = value; } }

		public Dictionary<string, Sound3d> Sounds3d { get { return m_oSounds3D; } set { m_oSounds3D = value; } }

		public Dictionary<string, Exception> ScenarioErrors { get { return m_oScenarioErrors; } set { m_oScenarioErrors = value; } }

		public bool NamesUpdated { get { return m_bNamesUpdated; } set { m_bNamesUpdated = value; } }


		public static void CheckRailroadsDirectory(string path)
		{
			if (!Directory.Exists(path))
			{
				throw new ArgumentException("Could not find the Sid Meier's Railroads! directory at '" + path + "'");
			}

			if (!Directory.Exists(path + Constants.ASSETS))
			{
				throw new ArgumentException("Could not find the Railroads! Assets directory.");
			}
		}


		public void Load()
		{
			ApplicationSettings oSettings = ManagerApplication.AppSettings;

			if (oSettings == null)
			{
				throw new ArgumentNullException("ApplicationSettings");
			}

			m_oScenarioErrors.Clear();

			LoadCommonLocations();

			CheckRailroadsDirectory(oSettings.RailroadsDirectory);

			CheckLocale(oSettings.Locale);

			LoadLocaleNames(oSettings);

			EntityLoader.LoadBaseEntities(this, oSettings);

			LoadCoreScenarios(oSettings);

			LoadCustomScenarios(oSettings);

			LoadUserScenarios(oSettings);

			LoadLocalScenarios(oSettings);
		}
		

		private void LoadLocaleNames(ApplicationSettings settings)
		{
			//Find the correct locale tagged items file
			string sPath = SearchCommonLocations(TAGGED_ITEMS_FILE + settings.Locale + ".xml");

			if (!File.Exists(sPath))
			{
				throw new ArgumentException("Could not find Names file at '" + sPath + "'");
			}

			XmlDocument oDoc = new XmlDocument();
			using (StreamReader oReader = new StreamReader(sPath, Encoding.UTF8))
			{
				oDoc.Load(oReader);
				oReader.Close();
			}

			//Load trains
			m_oBaseTrains = new Dictionary<string, Train>();
			List<Train> oTrains = LoadLocaleTrains(oDoc);
			foreach (Train oTrain in oTrains)
			{
				if (!m_oBaseTrains.ContainsKey(oTrain.Tag))
				{
					m_oBaseTrains.Add(oTrain.Tag, oTrain);
				}
			}

			//Load cities
			m_oBaseCities = new Dictionary<string, City>();
			List<City> oCities = LoadLocaleCities(oDoc);
			foreach (City oCity in oCities)
			{
				if (!m_oBaseCities.ContainsKey(oCity.Tag))
				{
					m_oBaseCities.Add(oCity.Tag, oCity);
				}
			}

			m_bNamesUpdated = false;
		}


		private List<Train> LoadLocaleTrains(XmlDocument doc)
		{
			List<Train> oTrains = new List<Train>();

			XmlNodeList oNodeList = doc.SelectNodes("/Names/RRTTrains/TextKey");
			if ((oNodeList != null) || (oNodeList.Count > 0))
			{
				foreach (XmlNode oNode in oNodeList)
				{
					Train oTrain = new Train();
					oTrain.FromXml(null, oNode);
					oTrains.Add(oTrain);
				}
			}

			return oTrains;
		}


		private List<City> LoadLocaleCities(XmlDocument doc)
		{
			List<City> oCities = new List<City>();

			XmlNodeList oNodeList = doc.SelectNodes("/Names/RRTCities/TextKey");
			if ((oNodeList != null) || (oNodeList.Count > 0))
			{
				foreach (XmlNode oNode in oNodeList)
				{
					City oCity = new City();
					oCity.FromXml(null, oNode);
					oCities.Add(oCity);
				}
			}

			return oCities;
		}


		private void LoadCoreScenarios(ApplicationSettings settings)
		{
			string sPath = settings.RailroadsDirectory + Constants.SCENARIO_PATH;
			if (!Directory.Exists(sPath))
			{
				throw new ArgumentException("Could not find core scenarios directory '" + sPath + "'");
			}

			m_oCoreScenarios = new Dictionary<string, Scenario>();

			LoadScenarioFiles(sPath, m_oCoreScenarios, ScenarioType.Core);
		}


		private void LoadCustomScenarios(ApplicationSettings settings)
		{
			string sPath = Locations.MyScenariosFolder;
			if (!Directory.Exists(sPath))
			{
				Directory.CreateDirectory(sPath);
			}

			m_oUserScenarios = new Dictionary<string, Scenario>();

			LoadScenarioFiles(sPath, m_oUserScenarios, ScenarioType.Custom);
		}


		private void LoadUserScenarios(ApplicationSettings settings)
		{
			string path = Locations.MyUserMapsFolder;
			if (!Directory.Exists(path))
			{
				Directory.CreateDirectory(path);
			}

			LoadScenarioFiles(path, m_oUserScenarios, ScenarioType.User);
		}


		private void LoadLocalScenarios(ApplicationSettings settings)
		{
			string sPath = settings.WorkingDirectory + Constants.SCENARIO_PATH;
			if (!Directory.Exists(sPath))
			{
				Directory.CreateDirectory(sPath);
			}

			m_oLocalScenarios = new Dictionary<string, Scenario>();
			LoadScenarioFiles(sPath, m_oLocalScenarios, ScenarioType.Local);
		}


		private void LoadScenarioFiles(string path, Dictionary<string, Scenario> scenarioList, ScenarioType type)
		{
			Scenario oScenario = null;
			string[] aSearch;

			//Look through each subdirectory of the scenarios folder and get the scenario name
			foreach (string sFolder in Directory.GetDirectories(path))
			{
				aSearch = Directory.GetFiles(sFolder, Constants.SCENARIO_SEARCH_PATTERN, SearchOption.AllDirectories);
				if ((aSearch != null) && (aSearch.Length > 0))
				{
					try
					{
						//Try to parse file as scenario xml file
						oScenario = Scenario.QuickLoad(aSearch[0]);
						oScenario.Type = type;
					}
					catch (Exception ex)
					{
						m_oScenarioErrors.Add(aSearch[0], ex);
					}

					//If file was parsed and is not already in the list then get the paths for the important files and add it to the list
					if ((oScenario != null) && !scenarioList.ContainsKey(oScenario.Tag))
					{
						scenarioList.Add(oScenario.Tag, oScenario);
					}
				}
			}
		}
		

		public void Save()
		{
			ApplicationSettings oSettings = ManagerApplication.AppSettings;
			if (m_bNamesUpdated)
			{
				SaveLocaleSettings(oSettings);
			}

			SaveLocalScenarios(oSettings);
		}


		private void SaveLocaleSettings(ApplicationSettings settings)
		{
			//Get locale settings path
			string sFile = SearchCommonLocations(TAGGED_ITEMS_FILE + settings.Locale + ".xml");

			//If current locale file is in railroads folder we'll make a copy in the CustomAssets folder
			if (Utility.IsSubdirectory(settings.RailroadsDirectory, sFile))
			{
				sFile = Locations.MyLocaleFolder + Path.GetFileName(sFile);
			}

			//Make sure directory exists
			if (!Directory.Exists(Path.GetDirectoryName(sFile)))
			{
				Directory.CreateDirectory(Path.GetDirectoryName(sFile));
			}

			//Delete the file as we're going totally regenerate it
			File.Delete(sFile);

			XmlDocument oDoc = CreateLocaleDocument();

			WriteXmlDocument(oDoc, sFile, Encoding.ASCII);
		}

		public static void WriteXmlDocument(XmlDocument document, string fileName, Encoding encoding)
		{
			MemoryStream oMemStream = new MemoryStream();

			XmlTextWriter oWriter = new XmlTextWriter(oMemStream, encoding);

			oWriter.Formatting = Formatting.Indented;
			oWriter.Indentation = 4;
			oWriter.IndentChar = ' ';

			document.Save(oWriter);

			oWriter.Flush();

			oMemStream.Position = 0;

			StreamReader oReader = new StreamReader(oMemStream);
			string sEverything = oReader.ReadToEnd();

			//"fix" <InCity></InCity>
			if (sEverything.Contains("<InCity> </InCity>"))
			{
				sEverything = sEverything.Replace("<InCity> </InCity>", "<InCity></InCity>");
			}

			StreamWriter oWriter2 = new StreamWriter(fileName);
			oWriter2.Write(sEverything);
			oWriter2.Flush();
			oWriter2.Close();

			oWriter.Close();
		}

		private XmlDocument CreateLocaleDocument()
		{
			XmlDocument oDoc = new XmlDocument();

			XmlDeclaration oXmlDeclaration = oDoc.CreateXmlDeclaration("1.0", "utf-8", null);
			oDoc.InsertBefore(oXmlDeclaration, oDoc.DocumentElement);

			XmlElement oElement = oDoc.CreateElement("Names");
			oDoc.AppendChild(oElement);

			//Add trains
			XmlElement oSubElement = oDoc.CreateElement("RRTTrains");
			oElement.AppendChild(oSubElement);
			foreach (KeyValuePair<string, Train> oPair in m_oBaseTrains)
			{
				oPair.Value.ToXml(oSubElement);
			}

			//Add cities
			oSubElement = oDoc.CreateElement("RRTCities");
			oElement.AppendChild(oSubElement);
			foreach (KeyValuePair<string, City> oPair in m_oBaseCities)
			{
				oPair.Value.ToXml(oSubElement);
			}
			return oDoc;
		}


		private void SaveLocalScenarios(ApplicationSettings settings)
		{
			string sPath = settings.WorkingDirectory + Constants.SCENARIO_PATH;
			if (!Directory.Exists(sPath))
			{
				Directory.CreateDirectory(sPath);
			}

			foreach (KeyValuePair<string, Scenario> oPair in m_oLocalScenarios)
			{
				if (oPair.Value.IsDirty)
				{
					if (string.IsNullOrEmpty(oPair.Value.Files.Scenario) || oPair.Value.CreateLocalFiles)
					{
						oPair.Value.Files = CreateLocalFiles(sPath, oPair.Value);
					}

					oPair.Value.Save();
				}
			}
		}


		private ScenarioFiles CreateLocalFiles(string path, Scenario scenario)
		{
			string sDir = path + "\\" + scenario.ScenarioSettings.ScenarioName + "\\";
			
			string sScenarioPath = sDir + Constants.SCENARIO_FILE + "_User_" + scenario.ScenarioSettings.ScenarioName + ".xml";
			
			string sCitiesPath = sDir + Path.GetFileName(scenario.ScenarioSettings.CitiesFile);
			string sNamesPath = sDir + Path.GetFileName(scenario.ScenarioSettings.NamesFile);
			string sGoodsPath = sDir + Path.GetFileName(scenario.ScenarioSettings.GoodsFile);
			string sTrainsPath = sDir + Path.GetFileName(scenario.ScenarioSettings.TrainsFile);
			string sIndustriesPath = sDir + Path.GetFileName(scenario.ScenarioSettings.IndustriesFile);
			string sDepotsPath = sDir + Path.GetFileName(scenario.ScenarioSettings.DepotsFile);
			string sTrainCarsPath = sDir + Path.GetFileName(scenario.ScenarioSettings.TrainCarsFile);
			string sEventsPath = sDir + Path.GetFileName(scenario.ScenarioSettings.EventsFile);
			string sErasPath = sDir + Path.GetFileName(scenario.ScenarioSettings.ErasFile);
			string sDecorationsPath = sDir + Path.GetFileName(scenario.ScenarioSettings.DecorationsFile);
			string sTreesPath = sDir + Path.GetFileName(scenario.ScenarioSettings.TreesFile);
			string sClutterPath = sDir + Path.GetFileName(scenario.ScenarioSettings.ClutterFile);
			string sBridgePath = sDir + Path.GetFileName(scenario.ScenarioSettings.BridgesFile);
			string sTunnelPath = sDir + Path.GetFileName(scenario.ScenarioSettings.TunnelsFile);
			string sQuaylePath = sDir + Path.GetFileName(scenario.ScenarioSettings.QuayleFile);

			return new ScenarioFiles(sScenarioPath, sCitiesPath, sNamesPath, sGoodsPath, sTrainsPath, sIndustriesPath, sDepotsPath,
				sTrainCarsPath, sEventsPath, sErasPath, sDecorationsPath, sTreesPath, sClutterPath, sBridgePath, sTunnelPath, sQuaylePath);
		}


		public void Create(Scenario scenario, ScenarioExistsActionType action)
		{
			if (scenario.Type != ScenarioType.Local)
			{
				return;
			}

			//Store the local scenario path to use later
			string sLocalScenarioPath = scenario.Files.Scenario;

			//Replace scenario name blanks with underscores (I don't know if this matters)
			string sEscapedScenarioName = scenario.ScenarioSettings.ScenarioName.Replace(" ", "_");

			//Create path for new scenario
			string sNewScenarioPath = CreateNewScenarioDirectory(scenario, action, sEscapedScenarioName);

			Create(scenario, sNewScenarioPath);
		}

		public void Create(Scenario scenario, string path)
		{
			try
			{
				//Replace scenario name blanks with underscores (I don't know if this matters)
				string sEscapedScenarioName = scenario.ScenarioSettings.ScenarioName.Replace(" ", "_");

				if (scenario.BaseScenario == null)
				{
					//Scenario has no base so just copy scenario files (support for export)
					CopyScenario(Path.GetDirectoryName(scenario.Files.Scenario), path);
				}
				else
				{
					//Copy the base scenario files to use for the new scenario
					CopyScenario(Path.GetDirectoryName(scenario.BaseScenario.Files.Scenario), path);
				}

				//Rename the copied files to use the file names defined in the new scenario
				scenario.Files = RenameScenarioFiles(scenario, path, sEscapedScenarioName);

				//Copy scenario map files
				scenario.ScenarioSettings.MapThumbnail = CopyScenarioMap(path, scenario.ScenarioSettings.MapThumbnail);
				scenario.ScenarioSettings.MapFile = CopyScenarioMap(path, scenario.ScenarioSettings.MapFile);

				//Update scenario lists
				//_localScenarios.Remove(scenario.Tag);
				scenario.Tag = scenario.Files.Scenario;
				m_oUserScenarios.Add(scenario.Tag, scenario);

				//This scenario is no longer dependant on the base scenario
				scenario.BaseScenario = null;
				scenario.Type = ScenarioType.Custom;

				//Save scenario to new location
				scenario.Save();
			}
			catch
			{
				if (!string.IsNullOrEmpty(path))
				{
					try
					{
						Utility.DeleteDirectory(path);
					}
					catch
					{
					}
				}

				throw;
			}
		}


		private string CreateNewScenarioDirectory(Scenario scenario, ScenarioExistsActionType action, string scenarioName)
		{
			string sNewScenarioPath = Locations.MyScenariosFolder + scenarioName;
			string sTemp = sNewScenarioPath;
			
			if (action == ScenarioExistsActionType.Replace)
			{
				if (Directory.Exists(sTemp))
				{
					Utility.DeleteDirectory(sTemp);
				}
			}
			else if (action == ScenarioExistsActionType.CreateNew)
			{
				//Make sure new scenario path does not already exist
				string sTempName = scenario.ScenarioSettings.ScenarioName;
				int i = 2;
				while (Directory.Exists(sTemp))
				{
					sTemp = sNewScenarioPath + "_" + i.ToString();
					sTempName = scenario.ScenarioSettings.ScenarioName + " " + i.ToString();
					i++;
				}

				scenario.ScenarioSettings.ScenarioName = sTempName;
			}

			return Utility.AddTrailingSlash(sTemp);
		}


		private void CopyScenario(string baseScenarioPath, string newScenarioPath)
		{
			//Create new scenario folder
			Directory.CreateDirectory(newScenarioPath);

			//Copy files from base scenario to new scenario
			Utility.CopyDirectory(baseScenarioPath, newScenarioPath, ".svn");
		}


		private ScenarioFiles RenameScenarioFiles(Scenario scenario, string exportPath, string scenarioName)
		{
			//Get current file paths
			ScenarioFiles sTempFiles;
			if (scenario.BaseScenario == null)
			{
				sTempFiles = FindScenarioFiles(exportPath, scenario);
			}
			else
			{
				sTempFiles = FindScenarioFiles(exportPath, scenario.BaseScenario);
			}

			//Rename the scenario file
			string sFile = exportPath + CreateScenarioFileName(scenario);
			File.Move(sTempFiles.Scenario, sFile);
			
			//Rename the cities file
			if (!string.IsNullOrEmpty(sTempFiles.Cities))
			{
				scenario.ScenarioSettings.CitiesFile = RenameFile(scenarioName, exportPath, sTempFiles.Cities, Constants.CITIES_FILE);
			}

			if (!string.IsNullOrEmpty(sTempFiles.Names))
			{
				scenario.ScenarioSettings.NamesFile = RenameFile(scenarioName, exportPath, sTempFiles.Names, Constants.NAMES_FILE);
			}

			//Rename the goods file
			if (!string.IsNullOrEmpty(sTempFiles.Goods))
			{
				scenario.ScenarioSettings.GoodsFile = RenameFile(scenarioName, exportPath, sTempFiles.Goods, Constants.GOODS_FILE);
			}

			//Rename trains file
			if (!string.IsNullOrEmpty(sTempFiles.Trains))
			{
				scenario.ScenarioSettings.TrainsFile = RenameFile(scenarioName, exportPath, sTempFiles.Trains, Constants.TRAINS_FILE);
			}

			//Rename industries file
			if (!string.IsNullOrEmpty(sTempFiles.Industries))
			{
				scenario.ScenarioSettings.IndustriesFile = RenameFile(scenarioName, exportPath, sTempFiles.Industries, Constants.INDUSTRIES_FILE);
			}

			//Rename depots file
			if (!string.IsNullOrEmpty(sTempFiles.Depots))
			{
				scenario.ScenarioSettings.DepotsFile = RenameFile(scenarioName, exportPath, sTempFiles.Depots, Constants.DEPOTS_FILE);
			}

			if (!string.IsNullOrEmpty(sTempFiles.TrainCars))
			{
				scenario.ScenarioSettings.TrainCarsFile = RenameFile(scenarioName, exportPath, sTempFiles.TrainCars, Constants.TRAIN_CARS_FILE);
			}

			if (!string.IsNullOrEmpty(sTempFiles.Events))
			{
				scenario.ScenarioSettings.EventsFile = RenameFile(scenarioName, exportPath, sTempFiles.Events, Constants.EVENTS_FILE);
			}

			if (!string.IsNullOrEmpty(sTempFiles.Eras))
			{
				scenario.ScenarioSettings.ErasFile = RenameFile(scenarioName, exportPath, sTempFiles.Eras, Constants.ERAS_FILE);
			}

			if (!string.IsNullOrEmpty(sTempFiles.Decorations))
			{
				scenario.ScenarioSettings.DecorationsFile = RenameFile(scenarioName, exportPath, sTempFiles.Decorations, Constants.DECORATIONS_FILE);
			}

			if (!string.IsNullOrEmpty(sTempFiles.Trees))
			{
				scenario.ScenarioSettings.TreesFile = RenameFile(scenarioName, exportPath, sTempFiles.Trees, Constants.TREES_FILE);
			}

			if (!string.IsNullOrEmpty(sTempFiles.Clutter))
			{
				scenario.ScenarioSettings.ClutterFile = RenameFile(scenarioName, exportPath, sTempFiles.Clutter, Constants.CLUTTER_FILE);
			}

			if (!string.IsNullOrEmpty(sTempFiles.Bridges))
			{
				scenario.ScenarioSettings.BridgesFile = RenameFile(scenarioName, exportPath, sTempFiles.Bridges, Constants.BRIDGES_FILE);
			}

			if (!string.IsNullOrEmpty(sTempFiles.Tunnels))
			{
				scenario.ScenarioSettings.TunnelsFile = RenameFile(scenarioName, exportPath, sTempFiles.Tunnels, Constants.TUNNELS_FILE);
			}

			if (!string.IsNullOrEmpty(sTempFiles.Quayle))
			{
				scenario.ScenarioSettings.QuayleFile = RenameFile(scenarioName, exportPath, sTempFiles.Quayle, Constants.QUAYLE_FILE);
			}

			sTempFiles = new ScenarioFiles(sFile, 
				scenario.ScenarioSettings.CitiesFile, 
				scenario.ScenarioSettings.NamesFile, 
				scenario.ScenarioSettings.GoodsFile, 
				scenario.ScenarioSettings.TrainsFile, 
				scenario.ScenarioSettings.IndustriesFile,
				scenario.ScenarioSettings.DepotsFile,
				scenario.ScenarioSettings.TrainCarsFile, 
				scenario.ScenarioSettings.EventsFile, 
				scenario.ScenarioSettings.ErasFile, 
				scenario.ScenarioSettings.DecorationsFile, 
				scenario.ScenarioSettings.TreesFile,
				scenario.ScenarioSettings.ClutterFile, 
				scenario.ScenarioSettings.BridgesFile, 
				scenario.ScenarioSettings.TunnelsFile, 
				scenario.ScenarioSettings.QuayleFile);

			scenario.ScenarioSettings.CitiesFile = Path.GetFileName(scenario.ScenarioSettings.CitiesFile);
			scenario.ScenarioSettings.NamesFile = Path.GetFileName(scenario.ScenarioSettings.NamesFile);
			scenario.ScenarioSettings.GoodsFile = Path.GetFileName(scenario.ScenarioSettings.GoodsFile);
			scenario.ScenarioSettings.TrainsFile = Path.GetFileName(scenario.ScenarioSettings.TrainsFile);
			scenario.ScenarioSettings.IndustriesFile = Path.GetFileName(scenario.ScenarioSettings.IndustriesFile);
			scenario.ScenarioSettings.DepotsFile = Path.GetFileName(scenario.ScenarioSettings.DepotsFile);
			scenario.ScenarioSettings.TrainCarsFile = Path.GetFileName(scenario.ScenarioSettings.TrainCarsFile);
			scenario.ScenarioSettings.EventsFile = Path.GetFileName(scenario.ScenarioSettings.EventsFile);
			scenario.ScenarioSettings.ErasFile = Path.GetFileName(scenario.ScenarioSettings.ErasFile);
			scenario.ScenarioSettings.DecorationsFile = Path.GetFileName(scenario.ScenarioSettings.DecorationsFile);
			scenario.ScenarioSettings.TreesFile = Path.GetFileName(scenario.ScenarioSettings.TreesFile);
			scenario.ScenarioSettings.ClutterFile = Path.GetFileName(scenario.ScenarioSettings.ClutterFile);
			scenario.ScenarioSettings.BridgesFile = Path.GetFileName(scenario.ScenarioSettings.BridgesFile);
			scenario.ScenarioSettings.TunnelsFile = Path.GetFileName(scenario.ScenarioSettings.TunnelsFile);
			scenario.ScenarioSettings.QuayleFile = Path.GetFileName(scenario.ScenarioSettings.QuayleFile);

			return sTempFiles;
		}


		private string RenameFile(string scenarioName, string exportPath, string entityFile, string initialEntityFileName)
		{
			StringBuilder oBuilder = new StringBuilder();

			oBuilder.Append(initialEntityFileName);
			oBuilder.Append("_");
			oBuilder.Append(scenarioName);
			oBuilder.Append(".xml");

			string sFile = exportPath + oBuilder.ToString();
			if (Utility.IsSubdirectory(ManagerApplication.AppSettings.RailroadsDirectory, Path.GetDirectoryName(entityFile)))
			{
				File.Copy(entityFile, sFile, true);
			}
			else
			{
				File.Move(entityFile, sFile);
			}

			return sFile;
		}


		private string CopyScenarioMap(string newScenarioPath, string file)
		{
			string sNewMapFile = FindMapFile(newScenarioPath, file);

			if (Path.GetDirectoryName(newScenarioPath) != Path.GetDirectoryName(sNewMapFile) && 
				!Utility.IsSubdirectory(newScenarioPath, sNewMapFile))
			{
				File.Copy(file, sNewMapFile, true);
			}

			//Delete any other map files
			string[] aSearch = Directory.GetFiles(newScenarioPath, "*" + Path.GetExtension(sNewMapFile));
			if ((aSearch != null) && (aSearch.Length > 1))
			{
				foreach (string sMapFile in aSearch)
				{
					if (sMapFile != sNewMapFile)
					{
						File.Delete(sMapFile);
					}
				}
			}

			return Path.GetFileName(sNewMapFile);
		}


		public string FindMapFile(string scenarioPath, string file)
		{
			string sDir = Path.GetDirectoryName(file);
			string[] aSearch;

			if (string.IsNullOrEmpty(sDir))
			{
				//Just file name so check to see if the file is already in the scenario folder
				aSearch = Directory.GetFiles(scenarioPath, file);
				if ((aSearch != null) && (aSearch.Length > 0))
				{
					//The file has already been copied into the new scenario folder so we're all set
					return aSearch[0];
				}
				else
				{
					file = SearchCommonLocations(file);
					if (string.IsNullOrEmpty(file))
					{
						throw new FileNotFoundException("Could not find the map file '" + file + "' in any common locations.");
					}
				}
			}

			//Either a full path to the file was defined or the file is not in the new scenario folder but was found somewhere else

			//Make sure the file exists
			if (!File.Exists(file))
			{
				throw new FileNotFoundException("Could not find the map file at '" + file + "'.");
			}

			return file;
		}


		private string CreateScenarioFileName(Scenario scenario)
		{
			StringBuilder oBuilder = new StringBuilder(50);
			
			oBuilder.Append(Constants.SCENARIO_FILE);
			oBuilder.Append("_");
			
			if (scenario.ScenarioSettings.IsMultiplayer)
			{
				oBuilder.Append("MP");
			}
			
			oBuilder.Append(m_oCoreScenarios.Count + m_oUserScenarios.Count + 1).ToString();
			oBuilder.Append("_");
			oBuilder.Append(scenario.ScenarioSettings.ScenarioName);
			
			oBuilder.Append(".xml");

			return oBuilder.ToString();
		}



		public void Import(string path, ScenarioExistsActionType action)
		{
			if (!File.Exists(path))
			{
				throw new FileNotFoundException("Could not find file at '" + path + "'");
			}

			string[] aSearch;
			string sImportPath = null;
			bool bDeletePath = false;

			try
			{
				if (CompressionUtility.IsZipFile(path))
				{
					sImportPath = ManagerApplication.AppSettings.WorkingDirectory + @"Export\" + Guid.NewGuid().ToString() + "\\";
					bDeletePath = true;

					Directory.CreateDirectory(sImportPath);

					CompressionUtility.DecompressFile(path, sImportPath);

					aSearch = Directory.GetFiles(sImportPath, Constants.SCENARIO_SEARCH_PATTERN);
					if ((aSearch == null) || (aSearch.Length == 0))
					{
						throw new ArgumentException("The zip file does not contain a valid scenario file.");
					}

					sImportPath = aSearch[0];
				}
				else
				{
					sImportPath = path;
				}

				Scenario oScenario;
				try
				{
					oScenario = Scenario.QuickLoad(sImportPath);
				}
				catch(Exception ex)
				{
					throw new ArgumentException("The selected file could not be imported because it is not a valid scenario.", ex);
				}

				string sNewScenarioPath = CreateNewScenarioDirectory(oScenario, action, oScenario.ScenarioSettings.ScenarioName.Replace(" ", "_"));

				//Copy scenario to CustomAssets folder
				Utility.CopyDirectory(Path.GetDirectoryName(sImportPath), sNewScenarioPath);

				//Rename scenario file
				RenameScenarioFile(oScenario, sNewScenarioPath);

				//Process names file
				aSearch = Directory.GetFiles(sNewScenarioPath, IMPORT_EXPORT_NAMES_FILE);
				if ((aSearch != null) && (aSearch.Length > 0))
				{
					ImportNames(aSearch[0]);
					File.Delete(aSearch[0]);
				}

				//Reload manager scenarios
				Load();
			}
			finally
			{
				if (bDeletePath)
				{
					try
					{
						Utility.DeleteDirectory(Path.GetDirectoryName(sImportPath));
					}
					catch
					{
					}
				}
			}
		}


		private void RenameScenarioFile(Scenario scenario, string path)
		{
			string sNewScenarioFileName = CreateScenarioFileName(scenario);

			string[] aSearch = Directory.GetFiles(path, Constants.SCENARIO_SEARCH_PATTERN);
			if ((aSearch != null) && (aSearch.Length > 0))
			{
				File.Move(aSearch[0], Utility.AddTrailingSlash(Path.GetDirectoryName(aSearch[0])) + sNewScenarioFileName);
			}
		}


		private void ImportNames(string file)
		{
			//Only update tagged items where they are new or different
			//Likewise only updat the names file if something was updated
			bool bUpdated = m_bNamesUpdated;
			
			XmlDocument oDoc = new XmlDocument();
			using (StreamReader oReader = new StreamReader(file, Encoding.UTF8))
			{
				oDoc.Load(oReader);
				oReader.Close();
			}

			//Load scenario cities
			List<City> oCities = LoadLocaleCities(oDoc);
			City oBaseCity;
			foreach (City oCity in oCities)
			{
				if (m_oBaseCities.TryGetValue(oCity.Tag, out oBaseCity))
				{
					if (bUpdated || (oBaseCity.Name != oCity.Name))
					{
						oBaseCity.Name = oCity.Name;
						bUpdated = true;
					}
				}
				else
				{
					m_oBaseCities.Add(oCity.Tag, oCity);
					bUpdated = true;
				}
			}

			//Load scenario trains
			List<Train> oTrains = LoadLocaleTrains(oDoc);
			Train oBaseTrain;
			foreach (Train oTrain in oTrains)
			{
				if (m_oBaseTrains.TryGetValue(oTrain.Tag, out oBaseTrain))
				{
					if (bUpdated || (oBaseTrain.Name != oTrain.Name))
					{
						oBaseTrain.Name = oTrain.Name;
						bUpdated = true;
					}
				}
				else
				{
					m_oBaseTrains.Add(oTrain.Tag, oTrain);
					bUpdated = true;
				}
			}

			if (bUpdated)
			{
				m_bNamesUpdated = true;

				//Save name changes
				SaveLocaleSettings(ManagerApplication.AppSettings);
			}
		}


		public void Export(Scenario scenario, string path, bool includeMaps)
		{
			string sStagePath = ManagerApplication.AppSettings.WorkingDirectory + @"Export\" + Guid.NewGuid().ToString() + "\\";

			ScenarioFiles oOriginilFiles = scenario.Files;

			try
			{
				if (scenario.State != ScenarioState.Loaded)
				{
					scenario.Load();
				}

				//Create scenario in stage folder
				Create(scenario, sStagePath);

				//Add names to stage folder
				ExportNames(scenario, sStagePath);

				if (!includeMaps)
				{
					RemoveMaps(sStagePath);
				}

				//Delete the export file if it already exists
				if (File.Exists(path))
				{
					File.Delete(path);
				}

				CompressionUtility.CompressFiles(new FileList(sStagePath + "*"), path);
			}
			finally
			{
				scenario.Files = oOriginilFiles;

				//Make sure stage directory is deleted
				Utility.DeleteDirectory(sStagePath);
			}
		}


		private void RemoveMaps(string stagePath)
		{
			string[] aMaps = Directory.GetFiles(stagePath, "*.tga");
			if ((aMaps != null) && (aMaps.Length > 0))
			{
				foreach (string sMap in aMaps)
				{
					File.Delete(sMap);
				}
			}

			aMaps = Directory.GetFiles(stagePath, "*.dds");
			if ((aMaps != null) && (aMaps.Length > 0))
			{
				foreach (string sMap in aMaps)
				{
					File.Delete(sMap);
				}
			}
		}


		private void ExportNames(Scenario scenario, string stageDir)
		{
			List<Train> oTrains = new List<Train>();
			List<City> oCities = new List<City>();

			foreach (KeyValuePair<string, ScenarioTrain> oPair in scenario.Trains)
			{
				oTrains.Add(m_oBaseTrains[oPair.Key]);
			}
			foreach (KeyValuePair<string, ScenarioCity> oPair in scenario.Cities)
			{
				oCities.Add(m_oBaseCities[oPair.Key]);
			}

			XmlDocument oDoc = new XmlDocument();
			XmlDeclaration oXmlDeclaration = oDoc.CreateXmlDeclaration("1.0", "utf-8", null);
			oDoc.InsertBefore(oXmlDeclaration, oDoc.DocumentElement);

			XmlElement oElement = oDoc.CreateElement("Names");
			oDoc.AppendChild(oElement);

			//Add scenario trains
			XmlElement oSubElement = oDoc.CreateElement("RRTTrains");
			oElement.AppendChild(oSubElement);
			foreach (Train oTrain in oTrains)
			{
				oTrain.ToXml(oSubElement);
			}

			//Add scenario cities
			oSubElement = oDoc.CreateElement("RRTCities");
			oElement.AppendChild(oSubElement);
			foreach (City oCity in oCities)
			{
				oCity.ToXml(oSubElement);
			}

			WriteXmlDocument(oDoc, stageDir + IMPORT_EXPORT_NAMES_FILE, Encoding.ASCII);
		}



		public bool ScenarioExists(Scenario scenario)
		{
			//Check to see if a scenario with this name already exists
			foreach (KeyValuePair<string, Scenario> oPair in m_oUserScenarios)
			{
				if (oPair.Value.ScenarioSettings.ScenarioName == scenario.ScenarioSettings.ScenarioName)
				{
					return true;
				}
			}

			return false;
		}


		public void RemoveScenario(Scenario scenario)
		{
			switch (scenario.Type)
			{
				case ScenarioType.Core:		m_oCoreScenarios.Remove(scenario.Tag); break;
				case ScenarioType.Custom:	m_oUserScenarios.Remove(scenario.Tag); break;
				case ScenarioType.Local:	m_oLocalScenarios.Remove(scenario.Tag); break;
			}
		}


		public Good GetBaseGood(string name)
		{
			if (name == Scenario.ANY_GOOD)
			{
				return ScenarioGood.AnyGood;
			}
			else if (name == Scenario.NO_GOOD)
			{
				return ScenarioGood.NoGood;
			}
			else
			{
				return m_oBaseGoods[name];
			}
		}


		public City AddBaseCity(string tag, string name)
		{
			//tag = tag.Replace(" ", "_");
			//tag = tag.ToUpper();

			City oCity = new City(tag, name);

			m_oBaseCities.Add(oCity.Tag, oCity);
			m_bNamesUpdated = true;

			return oCity;
		}


		public CityType GetBaseCityType(string name)
		{
			return m_oBaseCityTypes[name];
		}


		private void CheckLocale(string locale)
		{
			switch (locale)
			{
				case "DEU":
				case "ENU":
				case "ESP":
				case "FRA":
				case "ITA":
				case "NLD":
					return;

				default:
					throw new ArgumentException(locale + " is not a known locale.");
			}
		}


		public ScenarioFiles FindScenarioFiles(string dir, Scenario scenario)
		{
			string sScenarioFile = "";
			string[] aSearch = Directory.GetFiles(dir, Constants.SCENARIO_SEARCH_PATTERN, SearchOption.AllDirectories);
			if ((aSearch != null) && (aSearch.Length > 0))
			{
				sScenarioFile = aSearch[0];
			}

			string sCities = FindFile(dir, scenario.ScenarioSettings.CitiesFile);
			string sNames = FindFile(dir, scenario.ScenarioSettings.NamesFile);
			string sGoods = FindFile(dir, scenario.ScenarioSettings.GoodsFile);
			string sIndustries = FindFile(dir, scenario.ScenarioSettings.IndustriesFile);
			string sDepots = FindFile(dir, scenario.ScenarioSettings.DepotsFile);
			string sTrains = FindFile(dir, scenario.ScenarioSettings.TrainsFile);
			string sTrainCars = FindFile(dir, scenario.ScenarioSettings.TrainCarsFile);
			string sEvents = FindFile(dir, scenario.ScenarioSettings.EventsFile);
			string sEras = FindFile(dir, scenario.ScenarioSettings.ErasFile);
			string sDecorations = FindFile(dir, scenario.ScenarioSettings.DecorationsFile);
			string sTrees = FindFile(dir, scenario.ScenarioSettings.TreesFile);
			string sClutter = FindFile(dir, scenario.ScenarioSettings.ClutterFile);
			string sBridges = FindFile(dir, scenario.ScenarioSettings.BridgesFile);
			string sTunnels = FindFile(dir, scenario.ScenarioSettings.TunnelsFile);
			string sQuayle = FindFile(dir, scenario.ScenarioSettings.QuayleFile);

			return new ScenarioFiles(sScenarioFile, sCities, sNames, sGoods, sTrains, sIndustries, sDepots, sTrainCars, sEvents, sEras, 
				sDecorations, sTrees, sClutter, sBridges, sTunnels, sQuayle);
		}


		private string FindFile(string dir, string file)
		{
			if (string.IsNullOrEmpty(file)) return null;

			string[] aSearch = Directory.GetFiles(dir, file);
			if ((aSearch == null) || (aSearch.Length == 0))
			{
				return SearchCommonLocations(Path.GetFileName(file));
			}
			else
			{
				return aSearch[0];
			}
		}


		private void LoadCommonLocations()
		{
			m_aCommonLocations = new string[]
				{
					Locations.MyCustomAssetsFolder,
					ManagerApplication.AppSettings.RailroadsDirectory + Constants.ASSETS + "\\",
					ManagerApplication.AppSettings.WorkingDirectory + Constants.ASSETS + "\\"
				};
		}


		public string SearchCommonLocations(string file)
		{
			string[] aSearch;
			foreach (string sCommonLocation in m_aCommonLocations)
			{
				aSearch = Directory.GetFiles(sCommonLocation, file, SearchOption.AllDirectories);
				if ((aSearch != null) && (aSearch.Length > 0))
				{
					return aSearch[0];
				}
			}

			return null;
		}
	}
}