using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Xml;

namespace Railroads.ScenarioManager.Core
{
	public class BuildStuffCondition : Condition
	{
		private List<BuildThing> m_oBuildThings = new List<BuildThing>();


		public BuildStuffCondition() {  }
		public BuildStuffCondition(ScenarioSettings rootSetting) : base(rootSetting) { }


		protected override ConditionType OnGetConditionType() { return ConditionType.BuildStuff; }


		public class BuildThing : XmlSetting
		{
			private string			m_sObjectType = "Depot";
			private string			m_sTerminalType;
			private ScenarioCity	m_oCity;


			internal BuildThing()
			{
			}

			public BuildThing(ScenarioSettings rootSettings)
			{
				base.RootSetting = rootSettings;
			}

			protected Scenario Scenario { get { return (base.RootSetting as ScenarioSettings).Scenario; } }

			public string ObjectType { get { return m_sObjectType; } set { Scenario.SetProperty(ref m_sObjectType, value); } }
			public string TerminalType { get { return m_sTerminalType; } set { Scenario.SetProperty(ref m_sTerminalType, value); } }
			public ScenarioCity City { get { return m_oCity; } set { Scenario.SetProperty(ref m_oCity, value); } }


			public override void ToXml(XmlNode node)
			{
				XmlNode oBuildNode = Utility.CreateChildElement(node, "BuildListItem");

				Utility.SetChildElementValue(oBuildNode, "szObjectType", m_sObjectType);
				Utility.SetChildElementValue(oBuildNode, "szObjectName", m_sTerminalType);
				Utility.SetChildElementValue(oBuildNode, "szCityLocation", m_oCity.Tag);
			}


			public override void FromXml(Scenario scenario, XmlNode node)
			{
				ScenarioSettings oRoot = this.RootSetting as ScenarioSettings;
				if (oRoot == null)
				{
					throw new ArgumentException("Could not find parent scenario settings instance.");
				}

				m_sObjectType = Utility.GetChildElementValue<string>(node, "szObjectType", "Depot", false);
				m_sTerminalType = Utility.GetChildElementValue<string>(node, "szObjectName");

				string sCityTag = Utility.GetChildElementValue<string>(node, "szCityLocation");
				m_oCity = GetCity(oRoot.Scenario, sCityTag);
			}


			public override object Clone()
			{
				BuildThing oClone = new BuildThing();

				oClone.m_oCity = (ScenarioCity) m_oCity.Clone();
				oClone.m_sTerminalType = m_sTerminalType;

				return oClone;
			}


			public override bool Validate(List<string> errors)
			{
				return true;
			}
		}


		public List<BuildThing> BuildThings { get { return m_oBuildThings; } }


		internal override void SetRootSetting(XmlSetting root)
		{
			base.SetRootSetting(root);
			foreach (BuildThing oThing in m_oBuildThings)
			{
				oThing.RootSetting = root;
			}
		}


		protected override void ToXml_Internal(XmlNode node)
		{
			XmlNode oList = Utility.CreateChildElement(node, "BuildList");
			foreach (BuildThing oThing in m_oBuildThings)
			{
				oThing.ToXml(oList);
			}
		}


		public override void FromXml(Scenario scenario, XmlNode node)
		{
			base.FromXml(scenario, node);

			m_oBuildThings = new List<BuildThing>();

			XmlNodeList oBuildNodes = node.SelectNodes("BuildList/BuildListItem");
			if ((oBuildNodes != null) && (oBuildNodes.Count > 0))
			{
				foreach (XmlNode oBuildNode in oBuildNodes)
				{
					BuildThing thing = new BuildThing();
					thing.RootSetting = this.RootSetting;
					thing.FromXml(scenario, oBuildNode);
					m_oBuildThings.Add(thing);
				}
			}
		}


		public override bool Validate(List<string> errors)
		{
			if (m_oBuildThings.Count == 0)
			{
				errors.Add("You must define at least one thing to build.");
				return false;
			}

			return true;
		}


		protected override Condition Clone_Internal()
		{
			BuildStuffCondition oClone = new BuildStuffCondition();

			oClone.m_oBuildThings = new List<BuildThing>();
			foreach (BuildThing oThing in m_oBuildThings)
			{
				oClone.m_oBuildThings.Add((BuildThing) oThing.Clone());
			}

			return oClone;
		}


		protected override string DefaultSuccessBitmap { get { return "event_buildstuff.dds"; } }


		public override string ToString()
		{
			string s = "";

			for (int i = 0; i < m_oBuildThings.Count; i++)
			{
				BuildThing oThing = m_oBuildThings[i];
				if (s != "") s += ", ";
				s += oThing.TerminalType + " in " + oThing.City.ToString();
			}

			return "Build " + s;
		}
	}
}