using System;
using System.Diagnostics;
using System.Xml;

namespace Railroads.ScenarioManager.Core
{
	public abstract class Condition : XmlSetting
	{
		private string				m_sDescription;
		private string				m_sNewsTitle;
		private string				m_sNewsHeadline;
		private string				m_sNewsText;
		private string				m_sNewsBitmap;

		
		internal Condition()
		{
		}


		public Condition(ScenarioSettings rootSetting)
		{
			base.RootSetting = rootSetting;
		}


		protected abstract string DefaultSuccessBitmap { get; }


		protected Scenario Scenario { get { return (base.RootSetting as ScenarioSettings).Scenario; } }

		public ConditionType ConditionType { get { return OnGetConditionType(); } }
		protected abstract ConditionType OnGetConditionType();


		public string Description { get { return m_sDescription; } set { Scenario.SetProperty(ref m_sDescription, value); } }
		public string NewsTitle { get { return m_sNewsTitle; } set { Scenario.SetProperty(ref m_sNewsTitle, value); } }
		public string NewsHeadline { get { return m_sNewsHeadline; } set { Scenario.SetProperty(ref m_sNewsHeadline, value); } }
		public string NewsText { get { return m_sNewsText; } set { Scenario.SetProperty(ref m_sNewsText, value); } }
		public string NewsBitmap { get { return m_sNewsBitmap; } set { Scenario.SetProperty(ref m_sNewsBitmap, value); } }


		public static Condition CreateCondition(ScenarioSettings rootSettings, XmlNode conditionNode)
		{
			string sTemp = Utility.GetChildElementValue<string>(conditionNode, "szType");
			ConditionType oType = (ConditionType) Enum.Parse(typeof(ConditionType), sTemp);
			return CreateCondition(rootSettings, oType);
		}


		public static Condition CreateCondition(ScenarioSettings rootSettings, ConditionType type)
		{
			Condition oCondition = null;

			switch (type)
			{
				case ConditionType.BuildStuff:		oCondition = new BuildStuffCondition(rootSettings); break;
				case ConditionType.ConnectCities:	oCondition = new ConnectCondition(rootSettings); break;
				case ConditionType.GrowCity:		oCondition = new GrowCityCondition(rootSettings); break;
				case ConditionType.NetWorth:		oCondition = new NetWorthCondition(rootSettings); break;
				case ConditionType.OwnStock:		oCondition = new OwnStockCondition(rootSettings); break;
				case ConditionType.OwnStuff:		oCondition = new OwnStuffCondition(rootSettings); break;
				case ConditionType.ShipGoods:		oCondition = new ShipGoodsCondition(rootSettings); break;
				case ConditionType.SpeedRecord:		oCondition = new SpeedRecordCondition(rootSettings); break;
			}

			return oCondition;
		}


		public static ScenarioCity GetCity(Scenario scenario, string tag)
		{
			if (tag == Scenario.ANY_CITY)
			{
				return ScenarioCity.AnyCity;
			}
			else
			{
				ScenarioCity oCity;
				if (!scenario.Cities.TryGetValue(tag, out oCity))
				{
					scenario.Problems.Add(string.Format("Could not find a city with the tag '{0}' for scenario '{1}', replaced with 'ANY'.",
						tag, scenario.ScenarioSettings.ScenarioName));

					oCity = ScenarioCity.AnyCity;
				}

				return oCity;
			}
		}


		public static ScenarioCityType GetCityType(Scenario scenario, string name)
		{
			if (name == Scenario.ANY_CITY_TYPE)
			{
				return ScenarioCityType.AnyCityType;
			}
			else
			{
				ScenarioCityType oCityType;
				if (!scenario.CityTypes.TryGetValue(name, out oCityType))
				{
					scenario.Problems.Add(string.Format("Could not find a city type with the name '{0}' for scenario '{1}', replaced with 'ANY'.",
						name, scenario.ScenarioSettings.ScenarioName));

					return ScenarioCityType.AnyCityType;
				}

				return oCityType;
			}
		}


		public static ScenarioGood GetGood(Scenario scenario, string tag)
		{
			if (tag == Scenario.ANY_GOOD) return ScenarioGood.AnyGood;
			if (tag == Scenario.NO_GOOD) return ScenarioGood.NoGood;
			
			ScenarioGood oScenarioGood;
			if (!scenario.Goods.TryGetValue(tag, out oScenarioGood))
			{
				Good oGood;
				if (!Manager.Instance.BaseGoods.TryGetValue(tag, out oGood))
				{
					scenario.Problems.Add(string.Format("Could not find a good with the name '{0}' for scenario '{1}', replaced with 'ANY'.",
						tag, scenario.ScenarioSettings.ScenarioName));
					return ScenarioGood.AnyGood;
				}

				oScenarioGood = ScenarioGood.FromGood(oGood);
				scenario.Goods.Add(tag, oScenarioGood);
			}

			return oScenarioGood;
		}


		public static ScenarioIndustry GetIndustry(Scenario scenario, string tag)
		{
			ScenarioIndustry oIndustry;
			if (!scenario.Industries.TryGetValue(tag, out oIndustry))
			{
				if (scenario.Industries.Count > 0)
				{
					foreach (ScenarioIndustry oSomeIndustry in scenario.Industries.Values)
					{
						scenario.Problems.Add(string.Format("Could not find an industry with the name '{0}' for scenario '{1}', replaced with '{2}'.",
							tag, scenario.ScenarioSettings.ScenarioName, oSomeIndustry.Name));
						break;
					}


				}
				else 
				{
					throw new ArgumentException("Could not find an industry with the name '" + tag + "' for scenario " +
												scenario.ScenarioSettings.ScenarioName);
				}
			}

			return oIndustry;
		}


		public static ScenarioTrain GetTrain(Scenario scenario, string tag)
		{
			if (tag == Scenario.ANY_TRAIN) return ScenarioTrain.AnyTrain;

			ScenarioTrain oTrain;
			if (!scenario.Trains.TryGetValue(tag, out oTrain))
			{
				scenario.Problems.Add(string.Format("Could not find a scenario train with the tag '{0}' for scenario '{1}', replaced with 'ANY'.",
					tag, scenario.ScenarioSettings.ScenarioName));
				return ScenarioTrain.AnyTrain;
			}

			return oTrain;
		}


		protected abstract Condition Clone_Internal();
		protected abstract void ToXml_Internal(XmlNode node);


		internal virtual void SetRootSetting(XmlSetting root)
		{
			RootSetting = root;
		}

		public override void ToXml(XmlNode node)
		{
			XmlNode oCondNode = Utility.CreateChildElement(node, "Condition");

			Utility.SetChildElementValue(oCondNode, "szType", ConditionType);
			Utility.SetChildElementValue(oCondNode, "bRequired", 1);

			ToXml_Internal(oCondNode);

			Utility.SetChildElementValue(oCondNode, "szDescription", m_sDescription);
			Utility.SetChildElementValue(oCondNode, "szNewspaperTitleSuccess", m_sNewsTitle);
			Utility.SetChildElementValue(oCondNode, "szNewspaperHeadlineSuccess", m_sNewsHeadline);
			Utility.SetChildElementValue(oCondNode, "szNewspaperTextSuccess", m_sNewsText);
			Utility.SetChildElementValue(oCondNode, "szNewspaperBitmapSuccess", string.IsNullOrEmpty(m_sNewsBitmap) ? DefaultSuccessBitmap : m_sNewsBitmap);
		}


		public override void FromXml(Scenario scenario, XmlNode node)
		{
			m_sDescription = Utility.GetChildElementValue<string>(node, "szDescription");
			m_sNewsTitle = Utility.GetChildElementValue<string>(node, "szNewspaperTitleSuccess", "", false);
			m_sNewsHeadline = Utility.GetChildElementValue<string>(node, "szNewspaperHeadlineSuccess", "", false);
			m_sNewsText = Utility.GetChildElementValue<string>(node, "szNewspaperTextSuccess", "", false);
			m_sNewsBitmap = Utility.GetChildElementValue<string>(node, "szNewspaperBitmapSuccess", "", false);
		}


		public override object Clone()
		{
			Condition oClone = Clone_Internal();

			oClone.m_sDescription = m_sDescription;
			oClone.m_sNewsHeadline = m_sNewsHeadline;
			oClone.m_sNewsText = m_sNewsText;
			oClone.m_sNewsTitle = m_sNewsTitle;
			oClone.m_sNewsBitmap = m_sNewsBitmap;

			return oClone;
		}
	}
}