using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Xml;

namespace Railroads.ScenarioManager.Core
{
	public class GrowCityCondition : Condition
	{
		private List<GrowCity>			m_oCities = new List<GrowCity>();


		public GrowCityCondition() { }
		public GrowCityCondition(ScenarioSettings rootSetting) : base(rootSetting) { }


		protected override ConditionType OnGetConditionType() { return ConditionType.GrowCity; }


		public class GrowCity : XmlSetting
		{
			private ScenarioCityType	m_oInitialSize;
			private ScenarioCityType	m_oTargetSize;
			private ScenarioCity		m_oCity;
			private int					m_nGrowQuantity;


			internal GrowCity()
			{
			}


			public GrowCity(ScenarioSettings rootSettings)
			{
				base.RootSetting = rootSettings;
			}


			protected Scenario Scenario { get { return (base.RootSetting as ScenarioSettings).Scenario; } }


			public ScenarioCityType InitialSize { get { return m_oInitialSize; } set { Scenario.SetProperty(ref m_oInitialSize, value); } }
			public ScenarioCityType TargetSize { get { return m_oTargetSize; } set { Scenario.SetProperty(ref m_oTargetSize, value); } }
			public ScenarioCity City { get { return m_oCity; } set { Scenario.SetProperty(ref m_oCity, value); } }
			public int GrowQuantity { get { return m_nGrowQuantity; } set { Scenario.SetProperty(ref m_nGrowQuantity, value); } }

			
			public override void ToXml(XmlNode node)
			{
				XmlNode oCondNode = Utility.CreateChildElement(node, "GrowCityListItem");

				if (m_oInitialSize.Name != ScenarioCityType.AnyCityType.Name)
				{
					Utility.SetChildElementValue(oCondNode, "szInitialSize", m_oInitialSize.Name);
				}

				Utility.SetChildElementValue(oCondNode, "szTargetSize", m_oTargetSize.Name);
				Utility.SetChildElementValue(oCondNode, "szCityTagName", m_oCity.Tag);

				if (m_nGrowQuantity > 1)
				{
					Utility.SetChildElementValue(oCondNode, "iGrowQuantity", m_nGrowQuantity);
				}
			}


			public override void FromXml(Scenario scenario, XmlNode node)
			{
				ScenarioSettings oRoot = this.RootSetting as ScenarioSettings;
				if (oRoot == null)
				{
					throw new ArgumentException("Could not find parent scenario settings instance.");
				}

				string sTemp = Utility.GetChildElementValue<string>(node, "szTargetSize");
				m_oTargetSize = Condition.GetCityType(oRoot.Scenario, sTemp);

				string sCityTag = Utility.GetChildElementValue<string>(node, "szCityTagName");
				m_oCity = GetCity(oRoot.Scenario, sCityTag);
								
				sTemp = Utility.GetChildElementValue<string>(node, "szInitialSize", Scenario.ANY_CITY_TYPE, false);
				m_oInitialSize = Condition.GetCityType(oRoot.Scenario, sTemp);

				m_nGrowQuantity = Utility.GetChildElementValue<int>(node, "iGrowQuantity", -1, false);
			}


			public override object Clone()
			{
				GrowCity oClone = new GrowCity();

				oClone.m_oCity = (ScenarioCity)m_oCity.Clone();
				oClone.m_oInitialSize = (ScenarioCityType)m_oInitialSize.Clone();
				oClone.m_oTargetSize = (ScenarioCityType)m_oTargetSize.Clone();
				oClone.m_nGrowQuantity = m_nGrowQuantity;

				return oClone;
			}


			public override bool Validate(List<string> errors)
			{
				return true;
			}
		}



		public List<GrowCity> Cities { get { return m_oCities; } }


		internal override void SetRootSetting(XmlSetting root)
		{
			base.SetRootSetting(root);
			foreach (GrowCity oCity in m_oCities)
			{
				oCity.RootSetting = root;
			}
		}


		protected override void ToXml_Internal(XmlNode node)
		{
			XmlNode oCondNode = Utility.CreateChildElement(node, "GrowCityList");
			foreach (GrowCity oCity in m_oCities)
			{
				oCity.ToXml(oCondNode);
			}
		}


		public override void FromXml(Scenario scenario, XmlNode node)
		{
			base.FromXml(scenario, node);

			m_oCities = new List<GrowCity>();

			XmlNodeList oBuildNodes = node.SelectNodes("GrowCityList/GrowCityListItem");
			if ((oBuildNodes != null) && (oBuildNodes.Count > 0))
			{
				foreach (XmlNode oBuildNode in oBuildNodes)
				{
					GrowCity oCity = new GrowCity();
					oCity.RootSetting = this.RootSetting;
					oCity.FromXml(scenario, oBuildNode);
					m_oCities.Add(oCity);
				}
			}
		}


		public override bool Validate(List<string> errors)
		{
			if (m_oCities.Count == 0)
			{
				errors.Add("You must define at least one city to grow.");
				return false;
			}

			return true;
		}


		protected override Condition Clone_Internal()
		{
			GrowCityCondition oClone = new GrowCityCondition();

			oClone.m_oCities = new List<GrowCity>();
			foreach (GrowCity oCity in m_oCities)
			{
				oClone.m_oCities.Add((GrowCity) oCity.Clone());
			}

			return oClone;
		}


		protected override string DefaultSuccessBitmap { get { return "event_citygrowth.dds"; } }


		public override string ToString()
		{
			string s = "";

			foreach (GrowCity oCity in m_oCities)
			{
				if (s != "") s += ", ";

				if (oCity.GrowQuantity > 1)
				{
					s += oCity.GrowQuantity.ToString() + " " + ToPlural(oCity.InitialSize.Name) + " to " + ToPlural(oCity.TargetSize.Name);
				}
				else 
				{
					s += oCity.City.ToString() + " to " + oCity.TargetSize.Name;
				}
			}

			return "Grow " + s;
		}

		private string ToPlural(string name)
		{
			return (name.EndsWith("s")) ? name + "es" : name + "s";

		}
	}
}