using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Xml;

namespace Railroads.ScenarioManager.Core
{
	public class OwnStockCondition : Condition
	{
		private OwnerType		m_eOwner = OwnerType.Player;
		private int				m_nPercent = 100;


		public OwnStockCondition() { }
		public OwnStockCondition(ScenarioSettings rootSetting) : base(rootSetting) { }


		protected override ConditionType OnGetConditionType() { return ConditionType.OwnStock; }


		public OwnerType Owner { get { return m_eOwner; } set { Scenario.SetProperty(ref m_eOwner, value); } }
		public int Percent { get { return m_nPercent; } set { Scenario.SetProperty(ref m_nPercent, value); } }


		protected override void ToXml_Internal(XmlNode node)
		{
			Utility.SetChildElementValue(node, "szStockTarget", OwnerToString(m_eOwner));
			Utility.SetChildElementValue(node, "iPercentage", m_nPercent);
		}


		public override void FromXml(Scenario scenario, XmlNode node)
		{
			base.FromXml(scenario, node);

			string sOwner = Utility.GetChildElementValue<string>(node, "szStockTarget");
			m_eOwner = ParseOwner(sOwner);
			m_nPercent = Utility.GetChildElementValue<int>(node, "iPercentage");
		}


		public override bool Validate(List<string> errors)
		{
			if (m_nPercent == 0)
			{
				errors.Add("The percent of stock to own must be greater than zero.");
				return false;
			}

			return true;
		}


		public static OwnerType ParseOwner(string owner)
		{
			switch (owner)
			{
				case "ANY_STOCK_BUT_SELF":	return OwnerType.AnyOtherThanPlayer;
				case "ALL_STOCK_BUT_SELF":	return OwnerType.AllOtherThanPlayer;
				case "MY_STOCK":			return OwnerType.Player;
				default:					throw new ArgumentException("Unknown stock owner type '" + owner + "'");
			}
		}


		public static string OwnerToString(OwnerType type)
		{
			switch (type)
			{
				case OwnerType.Player:				return "MY_STOCK";
				case OwnerType.AnyOtherThanPlayer:	return "ANY_STOCK_BUT_SELF";
				case OwnerType.AllOtherThanPlayer:	return "ALL_STOCK_BUT_SELF";
				default:							return "";
			}
		}


		protected override Condition Clone_Internal()
		{
			OwnStockCondition oClone = new OwnStockCondition();

			oClone.m_eOwner = m_eOwner;
			oClone.m_nPercent = m_nPercent;

			return oClone;
		}


		protected override string DefaultSuccessBitmap { get { return (m_nPercent == 100) ? "event_buyout.dds" : "event_buystock.dds"; } }


		public override string ToString()
		{
			string s = "Shares";

			switch (m_eOwner)
			{
				case OwnerType.AllOtherThanPlayer:	s = m_nPercent + "% of all competitor shares"; break;
				case OwnerType.AnyOtherThanPlayer:	s = m_nPercent + "% of any competitor shares"; break;
				case OwnerType.Player:				s = m_nPercent + "% of your own shares"; break;
			}

			return "Own " + s;
		}
	}
}