using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Xml;

namespace Railroads.ScenarioManager.Core
{
	public class ShipGoodsCondition : Condition
	{
		private List<ShipGood> m_oGoods = new List<ShipGood>();


		public ShipGoodsCondition() { }
		public ShipGoodsCondition(ScenarioSettings rootSetting) : base(rootSetting) { }


		protected override ConditionType OnGetConditionType() { return ConditionType.ShipGoods; }


		public class ShipGood : XmlSetting
		{
			private Good			m_oGood;
			private ScenarioCity	m_oFromCity;
			private ScenarioCity	m_oToCity;
			private int				m_nCarloads;


			internal ShipGood()
			{
			}

			public ShipGood(ScenarioSettings rootSetting)
			{
				base.RootSetting = rootSetting;
			}

			protected Scenario Scenario { get { return (base.RootSetting as ScenarioSettings).Scenario; } }


			public Good Good { get { return m_oGood; } set { Scenario.SetProperty(ref m_oGood, value); } }
			public ScenarioCity FromCity { get { return m_oFromCity; } set { Scenario.SetProperty(ref m_oFromCity, value); } }
			public ScenarioCity ToCity { get { return m_oToCity; } set { Scenario.SetProperty(ref m_oToCity, value); } }
			public int Carloads { get { return m_nCarloads; } set { Scenario.SetProperty(ref m_nCarloads, value); } }


			public override void ToXml(XmlNode node)
			{
				XmlNode oCondNode = Utility.CreateChildElement(node, "GoodsListItem");

				Utility.SetChildElementValue(oCondNode, "szGood", m_oGood.Name);
				Utility.SetChildElementValue(oCondNode, "iCarloads", m_nCarloads);
				Utility.SetChildElementValue(oCondNode, "szCityFrom", m_oFromCity.Tag);
				Utility.SetChildElementValue(oCondNode, "szCityTo", m_oToCity.Tag);
			}


			public override void FromXml(Scenario scenario, XmlNode node)
			{
				ScenarioSettings oRoot = this.RootSetting as ScenarioSettings;
				if (oRoot == null)
				{
					throw new ArgumentException("Could not find parent scenario settings instance.");
				}

				string sTag = Utility.GetChildElementValue<string>(node, "szGood");
				m_oGood = GetGood(oRoot.Scenario, sTag);

				m_nCarloads = Utility.GetChildElementValue<int>(node, "iCarloads");

				sTag = Utility.GetChildElementValue<string>(node, "szCityFrom");
				m_oFromCity = GetCity(oRoot.Scenario, sTag);

				sTag = Utility.GetChildElementValue<string>(node, "szCityTo");
				m_oToCity = GetCity(oRoot.Scenario, sTag);
			}


			public override object Clone()
			{
				ShipGood result = new ShipGood();

				result.m_nCarloads = m_nCarloads;
				result.m_oFromCity = (ScenarioCity) m_oFromCity.Clone();
				result.m_oGood = (Good) m_oGood.Clone();
				result.m_oToCity = (ScenarioCity) m_oToCity.Clone();

				return result;
			}


			public override bool Validate(List<string> errors)
			{
				return true;
			}
		}


		public List<ShipGood> Goods { get { return m_oGoods; } }


		internal override void SetRootSetting(XmlSetting root)
		{
			base.SetRootSetting(root);

			foreach (ShipGood oGood in m_oGoods)
			{
				oGood.RootSetting = root;
			}
		}


		protected override void ToXml_Internal(XmlNode node)
		{
			XmlNode oCondNode = Utility.CreateChildElement(node, "GoodsList");
			foreach (ShipGood good in m_oGoods)
			{
				good.ToXml(oCondNode);
			}
		}


		public override void FromXml(Scenario scenario, XmlNode node)
		{
			base.FromXml(scenario, node);

			m_oGoods = new List<ShipGood>();

			XmlNodeList oNodes = node.SelectNodes("GoodsList/GoodsListItem");
			if ((oNodes != null) && (oNodes.Count > 0))
			{
				foreach (XmlNode oGoodNode in oNodes)
				{
					ShipGood oGood = new ShipGood();
					oGood.RootSetting = this.RootSetting;
					oGood.FromXml(scenario, oGoodNode);
					m_oGoods.Add(oGood);
				}
			}
		}


		public override bool Validate(List<string> errors)
		{
			if (m_oGoods.Count == 0)
			{
				errors.Add("You must define at least one good to ship.");
				return false;
			}

			return true;
		}


		protected override Condition Clone_Internal()
		{
			ShipGoodsCondition oClone = new ShipGoodsCondition();

			oClone.m_oGoods = new List<ShipGood>();
			foreach (ShipGood oGood in m_oGoods)
			{
				oClone.m_oGoods.Add((ShipGood) oGood.Clone());
			}

			return oClone;
		}


		protected override string DefaultSuccessBitmap { get { return "event_transportgoods.dds"; } }


		public override string ToString()
		{
			string s = "";

			foreach (ShipGood oGood in m_oGoods)
			{
				if (s != "") s += ", ";

				s += oGood.Carloads.ToString() + " carloads of " + oGood.Good.Name;

				if (oGood.FromCity != ScenarioCity.AnyCity)
				{
					s += " from " + oGood.FromCity.ToString();
				}

				if (oGood.ToCity != ScenarioCity.AnyCity)
				{
					s += " to " + oGood.ToCity.ToString();
				}
			}

			return "Transport " + s;
		}

	}
}