using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Xml;

namespace Railroads.ScenarioManager.Core
{
	public class SpeedRecordCondition : Condition
	{
		private SpeedRecordType		m_eRecordType;
		private ScenarioTrain		m_oTrain;
		private ScenarioCity		m_oFromCity;
		private ScenarioCity		m_oToCity;
		private ScenarioGood		m_oGood;
		private float				m_fSpeed;

		public SpeedRecordCondition()
		{
			m_oTrain = ScenarioTrain.AnyTrain;
			m_oFromCity = ScenarioCity.AnyCity;
			m_oToCity = ScenarioCity.AnyCity;
			m_oGood = ScenarioGood.AnyGood;
		}

		public SpeedRecordCondition(ScenarioSettings rootSetting) : this()
		{
			base.RootSetting = rootSetting;
		}


		protected override ConditionType OnGetConditionType() { return ConditionType.SpeedRecord; }


		public SpeedRecordType RecordType { get { return m_eRecordType; } set { Scenario.SetProperty(ref m_eRecordType, value); } }
		public ScenarioTrain Train { get { return m_oTrain; } set { Scenario.SetProperty(ref m_oTrain, value); } }
		public ScenarioCity FromCity { get { return m_oFromCity; } set { Scenario.SetProperty(ref m_oFromCity, value); } }
		public ScenarioCity ToCity { get { return m_oToCity; } set { Scenario.SetProperty(ref m_oToCity, value); } }
		public ScenarioGood Good { get { return m_oGood; } set { Scenario.SetProperty(ref m_oGood, value); } }
		public float Speed { get { return m_fSpeed; } set { Scenario.SetProperty(ref m_fSpeed, value); } }


		protected override void ToXml_Internal(XmlNode node)
		{
			Utility.SetChildElementValue(node, "szTrainTagName", m_oTrain.Tag);
			Utility.SetChildElementValue(node, "szStop1", m_oFromCity.Tag);
			Utility.SetChildElementValue(node, "szStop2", m_oToCity.Tag);

			if (m_oGood.Name == ScenarioGood.AnyGood.Name)
			{
				Utility.SetChildElementValue(node, "szGood", Scenario.ANY_GOOD);
			}
			else
			{
				Utility.SetChildElementValue(node, "szGood", m_oGood.Name);
			}

			if ((m_eRecordType == SpeedRecordType.SpecificSpeed) && (m_fSpeed > 0))
			{
				Utility.SetChildElementValue(node, "fSpeed", m_fSpeed);
			}
		}


		public override void FromXml(Scenario scenario, XmlNode node)
		{
			ScenarioSettings oRoot = this.RootSetting as ScenarioSettings;
			if (oRoot == null)
			{
				throw new ArgumentException("Could not find parent scenario settings instance.");
			}

			base.FromXml(scenario, node);

			string sTag = Utility.GetChildElementValue<string>(node, "szTrainTagName");
			m_oTrain = GetTrain(oRoot.Scenario, sTag);

			sTag = Utility.GetChildElementValue<string>(node, "szStop1");
			m_oFromCity = GetCity(oRoot.Scenario, sTag);

			sTag = Utility.GetChildElementValue<string>(node, "szStop2");
			m_oToCity = GetCity(oRoot.Scenario, sTag);

			sTag = Utility.GetChildElementValue<string>(node, "szGood");
			m_oGood = GetGood(oRoot.Scenario, sTag);

			XmlNode oSpeedNode = node.SelectSingleNode("fSpeed");
			if (oSpeedNode != null)
			{
				if (float.TryParse(oSpeedNode.InnerText, out m_fSpeed))
				{
					if (m_fSpeed > 0)
					{
						m_eRecordType = SpeedRecordType.SpecificSpeed;
					}
				}
			}
			else
			{
				m_eRecordType = SpeedRecordType.AnyRecord;
				m_fSpeed = 0f;
			}
		}


		public override bool Validate(List<string> errors)
		{
			if ((m_oTrain == null) || (m_oToCity == null) || (m_oFromCity == null) || (m_oGood == null)
				|| ((m_eRecordType == SpeedRecordType.SpecificSpeed) && (m_fSpeed == 0)))
			{
				errors.Add("The speed record is not valid.");
				return false;
			}
			else
			{
				return true;
			}
		}


		protected override Condition Clone_Internal()
		{
			SpeedRecordCondition oClone = new SpeedRecordCondition();

			oClone.m_oFromCity = (ScenarioCity) m_oFromCity.Clone();
			oClone.m_oGood = (ScenarioGood)m_oGood.Clone();
			oClone.m_eRecordType = m_eRecordType;
			oClone.m_fSpeed = m_fSpeed;
			oClone.m_oToCity = (ScenarioCity)m_oToCity.Clone();
			oClone.m_oTrain = (ScenarioTrain)m_oTrain.Clone();

			return oClone;
		}


		protected override string DefaultSuccessBitmap { get { return "event_speedrecord.dds"; } }


		public override string ToString()
		{
			string s = "Achieve speed record";

			switch (m_eRecordType)
			{ 
				case SpeedRecordType.AnyRecord:
					break;

				case SpeedRecordType.SpecificSpeed:
					s = "Achieve record with average speed of " + m_fSpeed.ToString() + " mph";
					break;
			}

			if (m_oTrain != ScenarioTrain.AnyTrain) s += " with " + m_oTrain.Name;
			if (m_oFromCity != ScenarioCity.AnyCity) s += " from " + m_oFromCity.ToString();
			if (m_oToCity != ScenarioCity.AnyCity) s += " to " + m_oToCity.ToString();

			if (m_oGood != ScenarioGood.AnyGood) s += " transporting " + m_oGood.ToString();

			return s;
		}
	}
}