using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Xml;

namespace Railroads.ScenarioManager.Core
{
	public class Era : XmlSetting
	{
		private List<Condition>	m_oConditions = new List<Condition>();
		private string			m_sEraName;
		private string			m_sDescription;
		private string			m_sVictoryText = "Congratulations!";
		private string			m_sFailureText = "You have failed to complete all the objectives.";
		private int				m_nStartYear;
		private int				m_nEndYear;
		private int				m_nStartMonth;
		private int				m_nEndMonth;



		internal Era()
		{
		}

		public Era(ScenarioSettings rootSetting)
		{
			base.RootSetting = rootSetting;
		}


		public List<Condition> Conditions { get { return m_oConditions; } }

		protected Scenario Scenario { get { return (base.RootSetting as ScenarioSettings).Scenario; } }

		public string EraName { get { return m_sEraName; } set { Scenario.SetProperty(ref m_sEraName, value); } }
		public string Description { get { return m_sDescription; } set { Scenario.SetProperty(ref m_sDescription, value); } }
		public string VictoryText { get { return m_sVictoryText; } set { Scenario.SetProperty(ref m_sVictoryText, value); } }
		public string FailureText { get { return m_sFailureText; } set { Scenario.SetProperty(ref m_sFailureText, value); } }
		public int StartYear { get { return m_nStartYear; } set { Scenario.SetProperty(ref m_nStartYear, value); } }
		public int EndYear { get { return m_nEndYear; } set { Scenario.SetProperty(ref m_nEndYear, value); } }
		public int StartMonth { get { return m_nStartMonth; } set { Scenario.SetProperty(ref m_nStartMonth, value); } }
		public int EndMonth { get { return m_nEndMonth; } set { Scenario.SetProperty(ref m_nEndMonth, value); } }


		public override void ToXml(XmlNode node)
		{
			XmlNode oEra = Utility.CreateChildElement(node, "Era");

			Utility.SetChildElementValue(oEra, "iEraBeginMonth", m_nStartMonth);
			Utility.SetChildElementValue(oEra, "iEraBeginYear", m_nStartYear);
			Utility.SetChildElementValue(oEra, "iEraEndMonth", m_nEndMonth);
			Utility.SetChildElementValue(oEra, "iEraEndYear", m_nEndYear);
			Utility.SetChildElementValue(oEra, "szEraName", m_sEraName);
			Utility.SetChildElementValue(oEra, "szEraDescription", m_sDescription);
			Utility.SetChildElementValue(oEra, "szEraVictoryBlurb", m_sVictoryText);
			Utility.SetChildElementValue(oEra, "szEraFailureBlurb", m_sFailureText);

			XmlNode oConditions = Utility.CreateChildElement(oEra, "VictoryConditions");

			foreach (Condition oCondition in m_oConditions)
			{
				oCondition.ToXml(oConditions);
			}
		}


		public override void FromXml(Scenario scenario, XmlNode node)
		{
			m_sEraName = Utility.GetChildElementValue<string>(node, "szEraName", "", false);
			m_sDescription = Utility.GetChildElementValue<string>(node, "szEraDescription", "", false);
			m_sVictoryText = Utility.GetChildElementValue<string>(node, "szEraVictoryBlurb");
			m_sFailureText = Utility.GetChildElementValue<string>(node, "szEraFailureBlurb");

			m_nStartMonth = Utility.GetChildElementValue<int>(node, "iEraBeginMonth");
			m_nStartYear = Utility.GetChildElementValue<int>(node, "iEraBeginYear");
			m_nEndMonth = Utility.GetChildElementValue<int>(node, "iEraEndMonth");
			m_nEndYear = Utility.GetChildElementValue<int>(node, "iEraEndYear");

			if (string.IsNullOrEmpty(m_sEraName))
			{
				m_sEraName = m_nStartYear.ToString() + " - " + m_nEndYear.ToString();
			}

			m_oConditions = new List<Condition>();

			XmlNodeList oConditionsNodes = node.SelectNodes("VictoryConditions/Condition");
			if ((oConditionsNodes != null) && (oConditionsNodes.Count > 0))
			{
				foreach (XmlNode oConditionNode in oConditionsNodes)
				{
					Condition oCondition = Condition.CreateCondition(Scenario.ScenarioSettings, oConditionNode);
					oCondition.RootSetting = this.RootSetting;

					if (oCondition != null)
					{
						oCondition.FromXml(scenario, oConditionNode);
						m_oConditions.Add(oCondition);
					}
				}
			}
		}


		public override object Clone()
		{
			Era oClone = new Era();

			oClone.m_sDescription = m_sDescription;
			oClone.m_nEndMonth = m_nEndMonth;
			oClone.m_nEndYear = m_nEndYear;
			oClone.m_sEraName = m_sEraName;
			oClone.m_sFailureText = m_sFailureText;
			oClone.m_nStartMonth = m_nStartMonth;
			oClone.m_nStartYear = m_nStartYear;
			oClone.m_sVictoryText = m_sVictoryText;

			oClone.m_oConditions = new List<Condition>();
			foreach (Condition oCondition in m_oConditions)
			{
				oClone.m_oConditions.Add((Condition) oCondition.Clone());
			}

			return oClone;
		}


		public override bool Validate(List<string> errors)
		{
			bool bResult = true;

			//Make sure the era starts before it ends
			string sErrorMsg = "Era '" + m_sEraName + "' ends before it begins.";
			if (m_nEndYear < m_nStartYear)
			{
				errors.Add(sErrorMsg);
				bResult = false;
			}
			else if (m_nEndYear == m_nStartYear)
			{
				if (m_nEndMonth < m_nStartMonth)
				{
					errors.Add(sErrorMsg);
					bResult = false;
				}
			}

			if (m_oConditions.Count == 0)
			{
				errors.Add("Era '" + m_sEraName + "' must have at least one condition.");
				bResult = false;
			}

			if (bResult)
			{
				foreach (Condition condition in m_oConditions)
				{
					if (!condition.Validate(errors))
					{
						bResult = false;
						break;
					}
				}
			}

			return bResult;
		}

		internal void SetRootSetting(XmlSetting root)
		{
			RootSetting = root;

			foreach (Condition oCcondition in m_oConditions)
			{
				oCcondition.SetRootSetting(root);
			}
		}
	}
}