using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Xml;

namespace Railroads.ScenarioManager.Core
{
	public class ScenarioCity : City
	{
		public static ScenarioCity AnyCity = new ScenarioCity(Scenario.ANY_CITY, "<Any City>");



		private List<CitySong>	m_oCitySongs = new List<CitySong>();

		private string			m_sType;
		private TrackSideType	m_eDefaultTrackSide;
		private double			m_dRotation;
		private int				m_nStartX;
		private int				m_nStartY;
		private bool			m_bIsStartLocation;




		public class CitySong : XmlSetting
		{
			private Sound3d			m_oSong;
			private List<string>	m_oEras = new List<string>();


			public CitySong()
			{
			}

			public CitySong(ScenarioSettings rootSettings)
			{
				base.RootSetting = rootSettings;
			}

			protected Scenario Scenario { get { return (base.RootSetting as ScenarioSettings).Scenario; } }

			public Sound3d Song { get { return m_oSong; } set { m_oSong = value; } }
			public List<string> Eras { get { return m_oEras; } set { m_oEras = value; } }


			public override void ToXml(XmlNode node)
			{
				XmlNode oSongNode = Utility.GetOrCreateChildElement(node, "CitySong[szSongID='" + m_oSong.SoundID + "']", "CitySong");

				Utility.SetChildElementValue(oSongNode, "szSongID", m_oSong.ScriptID);

				XmlNode oSongErasNode = Utility.GetOrCreateChildElement(oSongNode, "SongEras", "SongEras");
				oSongErasNode.RemoveAll();

				foreach (string oEra in m_oEras)
				{
					Utility.SetChildElementValue(oSongErasNode, "szSongEra", oEra);
				}
			}


			public override void FromXml(Scenario scenario, XmlNode node)
			{
				string sTemp = Utility.GetChildElementValue<string>(node, "szSongID");
				
				Sound3d oTempSong;
				if (Manager.Instance.Sounds3d.TryGetValue(sTemp, out oTempSong))
				{
					m_oSong = oTempSong;
				}

				m_oEras = new List<string>();
				XmlNodeList oList = node.SelectNodes("SongEras/szSongEra");
				if ((oList != null) && (oList.Count > 0))
				{
					foreach (XmlNode oXmlNode in oList)
					{
						m_oEras.Add(oXmlNode.InnerText);
					}
				}
			}


			public override object Clone()
			{
				CitySong oClone = new CitySong();

				oClone.m_oSong = (Sound3d)m_oSong.Clone();

				oClone.m_oEras = new List<string>();
				foreach (string oEra in m_oEras)
				{
					oClone.m_oEras.Add(oEra);
				}

				return oClone;
			}


			public override bool Validate(List<string> errors)
			{
				if (m_oEras.Count == 0)
				{
					errors.Add("Each city song must be defined for at least one era.");
					return false;
				}

				return true;
			}
		}


		public ScenarioCity()
		{
		}

		internal ScenarioCity(string tag, string name)
		{
			Tag = tag;
			Name = name;
		}

		public ScenarioCity(ScenarioSettings rootSettings) { base.RootSetting = rootSettings; }

		public List<CitySong> CitySongs { get { return m_oCitySongs; } set { m_oCitySongs = value; } }

		protected Scenario Scenario { get { return (base.RootSetting as ScenarioSettings).Scenario; } }

		public string Type { get { return m_sType; } set { Scenario.SetProperty(ref m_sType, value); } }
		public TrackSideType DefaultTrackSide { get { return m_eDefaultTrackSide; } set { Scenario.SetProperty(ref m_eDefaultTrackSide, value); } }
		public double Rotation { get { return m_dRotation; } set { Scenario.SetProperty(ref m_dRotation, value); } }
		public int StartX { get { return m_nStartX; } set { Scenario.SetProperty(ref m_nStartX, value); } }
		public int StartY { get { return m_nStartY; } set { Scenario.SetProperty(ref m_nStartY, value); } }
		public bool IsStartLocation { get { return m_bIsStartLocation; } set { Scenario.SetProperty(ref m_bIsStartLocation, value); } }


		public override void ToXml(XmlNode node)
		{
			XmlNode oCityNode = Utility.GetOrCreateChildElement(node, "City[szName='" + base.Tag + "']", "City");

			Utility.SetChildElementValue(oCityNode, "szName", base.Tag);
			Utility.SetChildElementValue(oCityNode, "Type", m_sType);
			Utility.SetChildElementValue(oCityNode, "StartX", m_nStartX);
			Utility.SetChildElementValue(oCityNode, "StartY", m_nStartY);
			Utility.SetChildElementValue(oCityNode, "Rotation", m_dRotation);
			Utility.SetChildElementValue(oCityNode, "bIsStartLocation", m_bIsStartLocation);
			Utility.SetChildElementValue(oCityNode, "szDefaultTrackSide", m_eDefaultTrackSide);

			Utility.RemoveElements(oCityNode, "CitySongs");

			if (m_oCitySongs.Count > 0)
			{
				XmlNode oCitySongsNode = Utility.GetOrCreateChildElement(oCityNode, "CitySongs", "CitySongs");
				foreach (CitySong oSong in m_oCitySongs)
				{
					oSong.ToXml(oCitySongsNode);
				}
			}
		}


		public override void FromXml(Scenario scenario, XmlNode node)
		{
			base.Tag = Utility.GetChildElementValue<string>(node, "szName");

			City oCity;
			if (scenario.Names.TryGetValue(base.Tag, out oCity))
			{
				base.Name = oCity.Name;
			}
			else 
			{
				if (Manager.Instance.BaseCities.TryGetValue(base.Tag, out oCity))
				{
					base.Name = oCity.Name;
				}
				else
				{
					//Could not find city tag so add it to manager names
					base.Name = "Unknown City";
					Manager.Instance.AddBaseCity(base.Tag, base.Name);
				}
			}

			m_sType = Utility.GetChildElementValue<string>(node, "Type");
			m_nStartX = Utility.GetChildElementValue<int>(node, "StartX");
			m_nStartY = Utility.GetChildElementValue<int>(node, "StartY");
			m_bIsStartLocation = Utility.GetChildElementValue<bool>(node, "bIsStartLocation");

			string sTemp = Utility.GetChildElementValue<string>(node, "szDefaultTrackSide");
			m_eDefaultTrackSide = (TrackSideType) Enum.Parse(typeof(TrackSideType), sTemp);

			m_dRotation = Utility.GetChildElementValue<double>(node, "Rotation");

			m_oCitySongs = base.LoadSubSetting<CitySong>(node, "CitySongs/CitySong");

			//Remove any city songs that were not found
			List<CitySong> oInvalidSongs = new List<CitySong>();
			foreach (CitySong oSong in m_oCitySongs)
			{
				if (oSong.Song == null)
				{
					oInvalidSongs.Add(oSong);
				}
			}
			foreach (CitySong oSong in oInvalidSongs)
			{
				m_oCitySongs.Remove(oSong);
			}
		}


		public override string ToString()
		{
			return base.Name;
		}


		public override object Clone()
		{
			ScenarioCity oClone = new ScenarioCity();

			oClone.Tag = Tag;
			oClone.Name = Name;

			oClone.m_eDefaultTrackSide = m_eDefaultTrackSide;
			oClone.m_bIsStartLocation = m_bIsStartLocation;
			oClone.m_dRotation = m_dRotation;
			oClone.m_nStartX = m_nStartX;
			oClone.m_nStartY = m_nStartY;
			oClone.m_sType = m_sType;

			oClone.m_oCitySongs = new List<CitySong>();
			foreach (CitySong oSong in m_oCitySongs)
			{
				oClone.m_oCitySongs.Add(oSong);
			}

			return oClone;
		}


		public override bool Validate(List<string> errors)
		{
			foreach (CitySong oSong in m_oCitySongs)
			{
				if (!oSong.Validate(errors))
				{
					return false;
				}
			}

			return true;
		}


		public void SetRootSetting(ScenarioSettings rootSetting)
		{
			base.RootSetting = rootSetting;
			foreach (CitySong oSong in m_oCitySongs)
			{
				oSong.RootSetting = rootSetting;
			}
		}
	}
}