using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Xml;

namespace Railroads.ScenarioManager.Core
{
	public class ScenarioIndustry : Industry
	{
		private List<Location> m_oLocations = new List<Location>();

		public ScenarioIndustry()
		{
		}

		public ScenarioIndustry(ScenarioSettings rootSettings)
		{
			base.RootSetting = rootSettings;
		}


		public class Location : XmlSetting
		{
			private ScenarioCity	m_oCity;
			private IndustrySize	m_eSize;
			private double			m_dRotation;
			private int				m_nStartX;
			private int				m_mStartY;

			
			public Location()
			{
			}

			public Location(ScenarioSettings rootSettings)
			{
				base.RootSetting = rootSettings;
			}

			protected Scenario Scenario { get { return (base.RootSetting as ScenarioSettings).Scenario; } }


			public ScenarioCity City { get { return m_oCity; } set { Scenario.SetProperty(ref m_oCity, value); } }
			public IndustrySize Size { get { return m_eSize; } set { Scenario.SetProperty(ref m_eSize, value); } }
			public double Rotation { get { return m_dRotation; } set { Scenario.SetProperty(ref m_dRotation, value); } }
			public int StartX { get { return m_nStartX; } set { Scenario.SetProperty(ref m_nStartX, value); } }
			public int StartY { get { return m_mStartY; } set { Scenario.SetProperty(ref m_mStartY, value); } }


			public override void ToXml(XmlNode node)
			{
				XmlNode oLocNode = Utility.CreateChildElement(node, "Location");

				Utility.SetChildElementValue(oLocNode, "StartX", m_nStartX);
				Utility.SetChildElementValue(oLocNode, "StartY", m_mStartY);
				Utility.SetChildElementValue(oLocNode, "Rotation", m_dRotation);
				Utility.SetChildElementValue(oLocNode, "Size", (int) m_eSize);

				Utility.SetChildElementValue(oLocNode, "InCity", (m_oCity != null) ? m_oCity.Tag : " ");
			}


			public override void FromXml(Scenario scenario, XmlNode node)
			{
				m_nStartX = Utility.GetChildElementValue<int>(node, "StartX");
				m_mStartY = Utility.GetChildElementValue<int>(node, "StartY");
				m_dRotation = Utility.GetChildElementValue<double>(node, "Rotation");

				int nTemp = Utility.GetChildElementValue<int>(node, "Size");
				if ((nTemp >= 0) && (nTemp <= 2))
				{
					m_eSize = (IndustrySize)nTemp;
				}

				string sTag = Utility.GetChildElementValue<string>(node, "InCity", "", false);
				if (sTag != null) sTag = sTag.Trim();
				if (!string.IsNullOrEmpty(sTag))
				{
					m_oCity = Condition.GetCity(this.Scenario, sTag);
				}
			}


			public override object Clone()
			{
				Location oClone = new Location();

				oClone.m_nStartX = m_nStartX;
				oClone.m_mStartY = m_mStartY;
				oClone.m_dRotation = m_dRotation;
				oClone.m_eSize = m_eSize;

				if (m_oCity != null)
				{
					oClone.m_oCity = (ScenarioCity)m_oCity.Clone();
				}

				return oClone;
			}


			public override bool Validate(List<string> errors)
			{
				return true;
			}
		}


		public List<Location> Locations { get { return m_oLocations; } }


		public override void ToXml(XmlNode node)
		{
			//New for RR 1.1
			Industry oBaseIndustry;
			if (!Manager.Instance.BaseIndustries.TryGetValue(base.Name, out oBaseIndustry))
			{
				//If the base industry cannot be found then it is a new industry for the scenario
				//and was loaded as a base industry.
				oBaseIndustry = this;
			}

			XmlNode oIndustryNode = base.ToXml(node, oBaseIndustry, true);

			if ((m_oLocations != null) && (m_oLocations.Count > 0))
			{
				XmlNode oLocationsNode = Utility.CreateChildElement(oIndustryNode, "Locations");
				foreach (Location oLocation in m_oLocations)
				{
					oLocation.ToXml(oLocationsNode);
				}
			}
		}


		public override void FromXml(Scenario scenario, XmlNode node)
		{
			//Find base industry
			string sIndustryName = Utility.GetChildElementValue<string>(node, "szName");

			Industry oBaseIndustry;
			if (Manager.Instance.BaseIndustries.TryGetValue(sIndustryName, out oBaseIndustry))
			{
				//New for RR 1.1
				//Only load base industry if one is defined

				//Copy base industry settings
				oBaseIndustry.CopyToScenarioIndusty(this);				
			}
			
			//Update with any defined settings
			base.FromXml(scenario, node);

			//Load locations
			m_oLocations = base.LoadSubSetting<Location>(node, "Locations/Location");
		}


		public override object Clone()
		{
			ScenarioIndustry oClone = new ScenarioIndustry();

			base.CopyToScenarioIndusty(oClone);

			oClone.m_oLocations = new List<Location>();
			foreach (Location oLocation in m_oLocations)
			{
				oClone.m_oLocations.Add((Location)oLocation.Clone());
			}

			return oClone;
		}


		internal void SetRootSetting(XmlSetting root)
		{
			base.RootSetting = root;

			foreach (Location oLocation in m_oLocations)
			{
				oLocation.RootSetting = root;
			}
		}
	}
}