using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Security.Permissions;
using System.Text;
using System.Xml;
using System.Globalization;
using System.Windows.Forms;

namespace Railroads.ScenarioManager.Core
{
	public static class Utility
	{
		public static T GetAttribute<T>(XmlNode infoNode, string attributeName, T defaultValue, bool required)
		{
			T oReturnValue = default(T);

			XmlAttribute oAttribute = infoNode.Attributes[attributeName];
			if (oAttribute == null)
			{
				if (!required)
				{
					//If attribute is not defined and is not required, return default value
					oReturnValue = defaultValue;
				}
				else
				{
					//If attritbute is not defined and is required thorw exception
					throw new ArgumentException("", attributeName);
				}
			}
			else
			{
				// if attribute is empty and not required, return default value
				if ((oAttribute.Value.Length == 0) && !required)
				{
					oReturnValue = defaultValue;
				}
				else
				{
					try
					{
						// capture attribute value
						string sNodeValue = oAttribute.Value;

						//Because there is no nice way to convert from a string to a T use Convert.ChangeType for all types
						oReturnValue = (T)Convert.ChangeType(sNodeValue, typeof(T));
					}
					catch (Exception)
					{
						if (!required)
						{
							oReturnValue = defaultValue;
						}
						else
						{
							throw;
						}
					}
				}
			}

			return oReturnValue;
		}


		public static T GetChildElementValue<T>(XmlNode node, string elementName)
		{
			return GetChildElementValue<T>(node, elementName, default(T), true, Application.CurrentCulture);
		}


		public static T GetChildElementValue<T>(XmlNode node, string elementName, T defaultValue, bool required)
		{
			return GetChildElementValue<T>(node, elementName, defaultValue, required, Application.CurrentCulture);
		}


		public static T GetChildElementValue<T>(XmlNode node, string elementName, T defaultValue, bool required, CultureInfo ci)
		{
			T oReturnValue = default(T);

			XmlNode oElement = node.SelectSingleNode(elementName);
			if (oElement == null)
			{
				if (!required)
				{
					//If attribute is not defined and is not required, return default value
					oReturnValue = defaultValue;
				}
				else
				{
					//If attritbute is not defined and is required throw exception
					throw new ArgumentException("", elementName);
				}
			}
			else
			{
				// if attribute is empty and not required, return default value
				if ((oElement.InnerText.Length == 0) && !required)
				{
					oReturnValue = defaultValue;
				}
				else
				{
					try
					{
						// capture attribute value
						string sNodeValue = oElement.InnerText;

						//Because there is no nice way to convert from a string to a T use Convert.ChangeType for all types
						oReturnValue = (T)Convert.ChangeType(sNodeValue, typeof(T), ci);
					}
					catch (Exception)
					{
						if (!required)
						{
							oReturnValue = defaultValue;
						}
						else
						{
							throw;
						}
					}
				}
			}

			return oReturnValue;
		}


		public static void SetChildElementValue(XmlNode node, string elementName, object value)
		{
			XmlNode oChild = node.SelectSingleNode(elementName);
			if (oChild == null)
			{
				oChild = node.OwnerDocument.CreateElement(elementName);
				node.AppendChild(oChild);
			}

			if (value != null)
			{
				if (value is bool)
				{
					oChild.InnerText = value.ToString().ToLower();
				}
				else if (value is float)
				{
					oChild.InnerText = ((float)value).ToString("#####0.000000");
				}
				else if (value is double)
				{
					oChild.InnerText = ((double)value).ToString("#####0.000000");
				}
				else 
				{
					string sTemp = value.ToString();
					if (sTemp != null)
					{
						oChild.InnerText = sTemp;
					}
				}
			}
		}


		public static XmlNode CreateChildElement(XmlNode node, string elementName)
		{
			XmlNode oChild = node.OwnerDocument.CreateElement(elementName);
			node.AppendChild(oChild);
			return oChild;
		}


		public static XmlNode GetOrCreateChildElement(XmlNode node, string path, string elementName)
		{
			XmlNode oModifiersNode = node.SelectSingleNode(path);
			if (oModifiersNode == null)
			{
				oModifiersNode = CreateChildElement(node, elementName);
			}

			return oModifiersNode;
		}


		public static void RemoveElements(XmlNode node, string elementName)
		{
			XmlNodeList oSearch = node.SelectNodes(elementName);
			if ((oSearch != null) && (oSearch.Count > 0))
			{
				foreach (XmlNode oFound in oSearch)
				{
					node.RemoveChild(oFound);
				}
			}
		}


		public static Color ToColor(string color)
		{
			try
			{
				if (color.Contains(","))
				{
					string[] sArgb = color.Split(new string[] {","}, StringSplitOptions.RemoveEmptyEntries);

					if (sArgb.Length == 3)
					{
						int nR = int.Parse(sArgb[0]);
						int nG = int.Parse(sArgb[1]);
						int nB = int.Parse(sArgb[2]);

						return Color.FromArgb(nR, nG, nB);
					}
					else if (sArgb.Length == 4)
					{
						int nA = int.Parse(sArgb[0]);
						int nR = int.Parse(sArgb[1]);
						int nG = int.Parse(sArgb[2]);
						int nB = int.Parse(sArgb[3]);

						return Color.FromArgb(nA, nR, nG, nB);
					}
				}
				else
				{
					return Color.FromName(color);
				}
			}
			catch
			{
			}

			throw new ArgumentException("The string '" + color + "' cannot be converted to a color.");
		}


		public static string ColorToRgbString(Color color)
		{
			return string.Format("{0},{1},{2}", color.R, color.G, color.B);
		}

		public static string AddTrailingSlash(string dir)
		{
			if (string.IsNullOrEmpty(dir)) return dir;

			if (!dir.EndsWith(Path.DirectorySeparatorChar.ToString()))
			{
				dir += Path.DirectorySeparatorChar.ToString();
			}

			return dir;
		}


		public static void CopyDirectory(string from, string to, params string[] excludes)
		{
			from = AddTrailingSlash(Path.GetFullPath(from));
			to = AddTrailingSlash(Path.GetFullPath(to));

			//Make sure source dir exists
			if (!Directory.Exists(from))
			{
				throw new DirectoryNotFoundException("The directory '" + from + "' could not be found.");
			}

			//Make sure we have permissions to read from source dir
			new FileIOPermission(FileIOPermissionAccess.Read, from).Demand();

			//Make sure the destination directory exists
			if (!Directory.Exists(to))
			{
				//If it doesn't create it
				Directory.CreateDirectory(to);
			}

			//Make sure we have the permissions to write to the destination dir
			new FileIOPermission(FileIOPermissionAccess.Write, to).Demand();

			//Copy the files in the root folder
			foreach (string sFile in Directory.GetFiles(from))
			{
				File.Copy(Path.GetFullPath(sFile), to + Path.GetFileName(sFile), true);
			}

			//Copy the sub-directories and sub-directory files to the same relative folders in the destination dir
			foreach (string sSubDir in GetChildDirectories(from, excludes))
			{
				if (!Directory.Exists(to + sSubDir))
				{
					Directory.CreateDirectory(to + sSubDir);
				}

				foreach (string sFile in Directory.GetFiles(from + sSubDir))
				{
					File.Copy(Path.GetFullPath(sFile), to + AddTrailingSlash(sSubDir) + Path.GetFileName(sFile), true);
				}
			}
		}


		public static string[] GetChildDirectories(string path, params string[] excludes)
		{
			List<string> oResults = new List<string>();

			path = AddTrailingSlash(path);
			int nPathLen = path.Length;

			if (excludes == null)
			{
				foreach (string sChild in Directory.GetDirectories(path, "*", SearchOption.AllDirectories))
				{
					oResults.Add(sChild.Substring(nPathLen));
				}
			}
			else
			{
				foreach (string child in Directory.GetDirectories(path, "*", SearchOption.AllDirectories))
				{
					string sTemp = child.Substring(nPathLen);
					foreach (string sExclude in excludes)
					{
						if ((sTemp == sExclude) || sTemp.StartsWith(AddTrailingSlash(sExclude)))
						{
							oResults.Add(sTemp);
							break;
						}
					}
				}
			}

			return oResults.ToArray();
		}


		public static void DeleteDirectory(string path)
		{
			if (Directory.Exists(path))
			{
				string[] aFiles = Directory.GetFiles(path, "*", SearchOption.AllDirectories);
				if ((aFiles != null) && (aFiles.Length > 0))
				{
					foreach (string sFile in aFiles)
					{
						File.Delete(sFile);
					}
				}

				Directory.Delete(path);
			}
		}


		public static bool IsSubdirectory(string baseDir, string subDir)
		{
			//Make sure both base and sub directories parameters are defined
			if (string.IsNullOrEmpty(baseDir) || string.IsNullOrEmpty(subDir))
			{
				return false;
			}

			baseDir = Path.GetDirectoryName(baseDir);
			subDir = Path.GetDirectoryName(subDir);

			//Make sure both base and sub directories are directories
			if (!Directory.Exists(baseDir) || !Directory.Exists(subDir))
			{
				return false;
			}

			//Get the absolute path for each dir
			baseDir = AddTrailingSlash(Path.GetFullPath(baseDir)).ToLower();
			subDir = AddTrailingSlash(Path.GetFullPath(subDir)).ToLower();

			if (subDir.Contains(baseDir))
			{
				string sChildDir = subDir.Substring(baseDir.Length);
				if (sChildDir.EndsWith("\\"))
				{
					sChildDir = sChildDir.Substring(0, sChildDir.Length - 1);
				}

				string[] aResults = Directory.GetDirectories(baseDir, sChildDir, SearchOption.AllDirectories);
				return (aResults.Length > 0);
			}
			else
			{
				return false;
			}
		}
	}
}